<?php
/**
* An example of a project-specific implementation.
*
* After registering this autoload function with SPL, the following line
* would cause the function to attempt to load the \Foo\Bar\Baz\Qux class
* from /path/to/project/src/Baz/Qux.php:
*
*      new \Foo\Bar\Baz\Qux;
*
* @param string $class The fully-qualified class name.
* @return void
*/

spl_autoload_register(

/**
 * @param $class
 */
    function ($class)
    {

        $paths = array(
            array(
                'prefix'    => 'hji\\common\\',
                'base_dir'  => __DIR__ . '/common/'
            ),
            array(
                'prefix'    => 'hji\\membership\\thirdparty\\',
                'base_dir'  => __DIR__ . '/third-party/'
            ),
            array(
                'prefix'    => 'hji\\membership\\',
                'base_dir'  => __DIR__ . '/classes/'
            )
        );

        // Check if current class is within any of the given namespaces

        foreach ($paths as $path)
        {
            $len = strlen($path['prefix']);
            $x   = strncmp($path['prefix'], $class, $len);

            // If so, assign values to the expected variables

            if ($x == 0)
            {
                $prefix = $path['prefix'];
                $base_dir = $path['base_dir'];
                break;
            }
        }

        // Stop if variables are not defined

        if (!isset($prefix) || !isset($base_dir))
        {
            return;
        }

        // get the relative class name
        $relative_class = substr($class, $len);

        // replace the namespace prefix with the base directory, replace namespace
        // separators with directory separators in the relative class name, append
        // with .php
        $file = $base_dir . str_replace('\\', '/', $relative_class) . '.php';

        // if the file exists, require it
        if (file_exists($file))
        {
            require $file;
        }
    }
);