<?php

namespace hji\membership\controllers;

use hji\membership\Membership;
use hji\membership\models\compliances\ComplianceBase;
use hji\membership\utils\ComplianceGuard;


class ComplianceController
{
    /**
     * @var array
     */
    protected $markets;

    /**
     * @var array
     */
    protected $requirements;

    /**
     * @var
     */
    public $guard;

    public function __construct()
    {
        $compliance = new ComplianceBase();

        $this->markets = Membership::getInstance()->getMarkets();
        $this->requirements = $compliance->getMarketsRequirements();
        $this->guard = new ComplianceGuard();
    }

    /**
     * Loads the logic of the compliance stuff
     */
    public function init()
    {
        if (true === $this->check()) {
            $classes = [];

            // lets' build a list of compliance classes we need to load
            // but we don't want to new them up more than once...so we reassign
            foreach ($this->markets as $market) {
                if (isset($this->requirements[$market])) {
                    $classes[$this->requirements[$market]] = $this->requirements[$market];
                }
            }

            foreach ($classes as $class) {
                new $class;
            }

            return true;
        }

        return false;
    }


    /**
     * Checks to see if we should load compliance
     *
     * @return bool
     */
    private function check()
    {
        return ($this->markets && (count(array_intersect($this->markets, $this->requirements)) === 0));
    }
}
