<?php
namespace hji\membership\controllers;


use hji\membership\factories\ProductFactory;
use hji\membership\Membership;


/**
 * Class ProductRegistrar
 *
 * @since 2.0
 * @package hji\membership\controllers
 */
class ProductRegistrar
{
    private static $_instance;

    /**
     * Registered products collection
     *
     * @var
     */
    protected static $collection = array();


    static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    /**
     * Adds product object to the collection array
     *
     * @param      $file - absolute path to the product's main file
     * @param      $type - product type (plugin, theme)
     * @param null $callback - callback function for membership to initialize the product
     */
    public function register($file, $type, $callback = null)
    {
        if (!$this->isRegistered($file))
        {
            self::$collection[] = new RegistryEntry($file, $type, $callback);
        }
    }


    /**
     * Retrieves registry entry by file
     *
     * @param $file
     * @return mixed
     */
    public function getEntry($file)
    {
        return $this->isRegistered($file);
    }


    public function getEntries()
    {
        return self::$collection;
    }


    /**
     * Removes product object from the collection array
     *
     * @param $file
     */
    public function deregister($file)
    {
        if (!empty(self::$collection))
        {
            foreach (self::$collection as $key => $entry)
            {
                if ($entry->file == $file)
                {
                    unset(self::$collection[$key]);
                }
            }
        }
    }


    /**
     * Determines if product exists in the collection
     *
     * @param $file
     * @return bool
     */
    public function isRegistered($file)
    {
        if (!empty(self::$collection))
        {
            foreach (self::$collection as $entry)
            {
                if ($entry->file == $file)
                {
                    return $entry;
                }
            }
        }

        return false;
    }
}


/**
 * Class RegistryEntry
 *
 * @since 2.0
 * @package hji\membership\controllers
 */
class RegistryEntry
{
    protected $file;

    protected $type;

    protected $callback;

    protected $status = 0;


    function __construct($file, $type, $callback = null)
    {
        $this->file     = $file;
        $this->callback = $callback;
        $this->type     = $type;

        $productFactory = new ProductFactory();
        $this->product  = $productFactory->create($this->file, $this->type);
    }


    public function __get($property)
    {
        if (property_exists(__CLASS__, $property))
        {
            return $this->$property;
        }
    }


    public function setStatus($status)
    {
        $this->status = $status;
    }


    public function getStatus()
    {
        return $this->status;
    }
}