<?php
namespace hji\membership\controllers;


use hji\membership\Membership;


class RequireJs
{
    private static $_instance;

    private static $paths = [];
    private static $versions = [];

    private static $settingsPrinted = false;

    private function __construct()
    {
//        add_action('wp_print_scripts', array($this, 'loadScripts'), -10);
        // Font-end

//        add_action('wp_footer', array($this, 'renderInHeader'), 100);
        // Back-end
        if (is_admin())
        {
            add_action('admin_print_scripts', array($this, 'renderInHeader'));
        }
        else
        {
            add_action('wp_print_scripts', array($this, 'renderInHeader'), -10);
        }

        add_action('wp_enqueue_scripts', array($this, 'wp_enqueue_amd_scripts'), 9999);
    }


    static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    public function loadScripts()
    {
//        wp_enqueue_script('hji-requirejs', Membership::$url . '/third-party/requirejs/require.js', array('jquery', 'underscore', 'backbone'), false, false);
    }


    public function renderInHeader()
    {
        if (self::$settingsPrinted)
        {
            return;
        }

        $obj = new \stdClass();
        $obj->paths = self::$paths;
        $obj->versions = self::getVersions();
        ?>
        <script>
        var hjiRequireJsConfig = {
            is_wp_admin: <?php echo (is_admin()) ? 'true' : 'false'; ?>,
            baseUrl: '<?php echo wp_make_link_relative(Membership::$url . '/resources/scripts/build'); ?>',
            waitSeconds: 30,
            paths: <?php echo json_encode($obj->paths); ?>,
            shim: {
                'backbone': {
                    deps: ['jquery', 'underscore'],
                    exports: 'Backbone'
                },
                'underscore': {
                    exports: '_'
                },
                'selectize': ['jquery']
            },
            map: {
                '*': {
                    'Backbone' : 'backbone',
                    '_' : 'underscore'
                }
            },
            urlArgs: function(id, url) {
                var versions = <?php echo json_encode($obj->versions); ?>,
                    ver = null,
                    param = '';

                // we already have the proper ID, so use it
                if (typeof versions[id] != 'undefined')
                {
                    ver = versions[id];
                }
                else // we need to regex the plugin slug from the script url and pass it to versions
                {
                    var re = new RegExp(/\/(hji-[a-z,-]+)\//gi),
                        match = re.exec(url);

                    if (match && (typeof match[1] != 'undefined') && (versions[match[1]]))
                    {
                        ver = versions[match[1]];
                    }
                }

                // if we actually have a version number add it as a URL param
                if (ver)
                {
                   param = (url.indexOf('?') === -1 ? '?' : '&') + ('ver=' + ver);
                }

                return param;
            }
        };

        </script>
    <?php

        self::$settingsPrinted = true;
    }


    public function getPaths()
    {
        return self::$paths;
    }


    public function addPath($key, $path, $external = false)
    {
        if (!isset(self::$paths[$key]))
        {
            if (!$external)
            {
                $path = wp_make_link_relative($path);

                self::$paths[$key] = $path;
            }
            else
            {
                self::$paths[$key] = $path;
            }
        }
    }


    /**
     * Intercepts registered scripts
     * checks if they have require-js as dependency
     * and if so, registers them with RequireJs instead.
     */
    function wp_enqueue_amd_scripts()
    {
        global $wp_scripts;

        if (empty($wp_scripts->registered))
        {
            return;
        }

        $requireJs = \hji\membership\Membership::getInstance()->requireJs;

        foreach ($wp_scripts->registered as $handleObject)
        {
            // We're only interested in scripts that have 'require-js' as their dependency

            if (in_array('require-js', $handleObject->deps))
            {
                $exceptions = array('hji-membership');
                $exceptions = apply_filters('hji-requirejs-exceptions', $exceptions);

                if (in_array($handleObject->handle, $exceptions))
                {
                    continue;
                }

                if ($this->_isExternal($handleObject->src))
                {
                    $requireJs->addPath($handleObject->handle, $handleObject->src, true);
                }
                else
                {
                    $requireJs->addPath($handleObject->handle, str_replace('.js', '', $handleObject->src));
                }

                $requireJs->addVersion($handleObject->handle, $handleObject->ver);

                $wp_scripts->dequeue($handleObject->handle);
            }
        }

    }


    /**
     * Determines if current script is MAIN,
     * and changes path to regular non-built file if WP_DEBUG mode is enabled.
     *
     * @param $key      RequireJS module key
     * @param $path     RequireJS module path
     * @return mixed    Module path
     */
    function maybeDevPath($key, $path)
    {
        _deprecated_function(__FUNCTION__, '3.0.6');
    }


    function _fileExists($nonBuilt)
    {
        $filePath = untrailingslashit(ABSPATH) . $nonBuilt . '.js';

        // Check if main.js exists

        if (file_exists($filePath))
        {
            return true;
        }
    }


    function _isExternal($path)
    {
        $external = false;

        if (substr($path, 0, 7) == 'http://'  ||
            substr($path, 0, 8) == 'https://' ||
            substr($path, 0, 2) == '//'
        )
        {
            $external = true;
        }

        return $external;
    }


    /**
     * Adds version numbers to the static property for use with the requireJS urlArgs callback
     *
     * @since 2.7.0
     * @param $key     string the script handle
     * @param $version string the version number to add to the URL as a param
     */
    public function addVersion($key, $version)
    {
        if (($key != null) && ($version != null))
        {
            self::$versions[$key] = $version;
        }
    }


    /**
     * Grabs our plugin versions and appends the version number of the plugin to an object param
     *
     * @since 2.7.0
     */
    protected function getVersions()
    {
        $requireJs = \hji\membership\Membership::getInstance()->requireJs;
        $products = ProductRegistrar::getInstance()->getEntries();

        foreach ($products as $product)
        {
            if ($product->type == 'plugin')
            {
                $requireJs::addVersion($product->product->getSlug(), $product->product->getData()['Version']);
            }
        }

        return self::$versions;
    }

}
