<?php
namespace hji\membership\controllers;


use \hji\common\utils\Updater;
use \hji\membership\models\WpCatalog;


/**
 * Class UpdateManager
 *
 * @since 2.0
 * @package hji\membership\controllers
 */
class UpdateManager
{
    private static $_instance;

    protected $registeredProducts;

    protected $collection;


    private function __construct()
    {
        $this->initActionHooks();
    }


    public function scheduleUpdates($product)
    {
        $updater = new Updater($product);

        $slug = $updater->slug;
        $this->collection[$slug] = $updater;
    }


    function initActionHooks()
    {
        add_filter('pre_http_request', array($this, 'interceptUpdateCalls'), 100, 3);
    }


    static function getInstance()
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    /**
     * Preemptive update response to avoid unnecessary API calls
     *
     * Implementation of pre_http_request hook.
     * Intercepting request URL, determining it it's an HJI plugin update call,
     * injecting response from already cached catalog.
     *
     * @since 2.0.0
     * @param $return
     * @param $r
     * @param $url
     * @return mixed
     */
    function interceptUpdateCalls($return, $r, $url)
    {
        // We're getting is_user_logged_in() does not exist from time to time,
        // which causes site to die with 500 code. Therefore we include pluggable.php

        if (!function_exists('is_user_logged_in'))
        {
            require_once ABSPATH . WPINC . '/pluggable.php';
        }

        if ($slug = $this->getProductSlug($url))
        {
            if ($data = $this->fromCatalog('plugins', $slug))
            {
                $return = $this->_fakeResponse($data);
            }
            else if ($data = $this->fromCatalog('themes', $slug))
            {
                $return = $this->_fakeResponse($data);
            }
        }

        return $return;
    }


    function _fakeResponse($data)
    {
        // fake api response
        $result['response']['code'] = '200';
        $result['body'] = json_encode($data);

        return $result;
    }


    /**
     * Returns plugin data from already cached catalog
     * instead of making a separate info call
     *
     * @since 2.0.0
     * @param $type - string plugins|themes
     * @param $slug - product slug
     * @return mixed
     */
    function fromCatalog($type, $slug)
    {
        if (is_admin())
        {
            $catalog = WpCatalog::browse();
        }
        else
        {
            $catalog = array();
        }

        if (isset($catalog[$type]) && !empty($catalog[$type]))
        {
            foreach ($catalog[$type] as $product)
            {
                if ($product['slug'] == $slug)
                {
                    return $product;
                }
            }
        }
    }


    /**
     * Returns product slug from update request URL
     *
     * @param $url
     * @return bool
     */
    function getProductSlug($url)
    {
        $parsedUrl = parse_url($url);

        if (isset($parsedUrl['query']))
        {
            parse_str($parsedUrl['query'], $query);

            // Updater passes extra parameter to the URL - 'slug'
            // - see if it matches any scheduled updates in the collection

            if (isset($query['slug']))
            {
                return $query['slug'];
            }
        }

        return false;
    }


    public function getUpdater($slug)
    {
        if (isset($this->collection[$slug]))
        {
            return $this->collection[$slug];
        }
    }
} 