<?php
namespace hji\membership\extension\abstracts;

use hji\membership\abstracts\Component as MembershipComponent;
use hji\membership\extension\interfaces\ExtensionInterface;


/**
 * Class Component to use for extending components, controllers and models in other plugins
 *
 * @codeCoverageIgnore
 * @package hji\membership\extension\abstracts
 */
abstract class Component extends MembershipComponent implements ExtensionInterface
{
    public function membershipServices()
    {
        return parent::getServiceContainer();
    }


    /**
     * Method is intended to be overwritten by a local service container.
     * If not overwritten, it will look for hji\your-plugin\ServiceContainer class.
     * Otherwise, it will return membershipServices().
     *
     * Example:
     *
    $servicesClassName = 'hji\\project\\services\\ServiceContainer';

    // Always return new instance
    return $this->di()->create($servicesClassName);

    // Always return the same instance
    return $this->di()->get($servicesClassName);
     */
    public function services()
    {
        $className = get_class($this);
        $parts = explode('\\', $className);

        // All hji plugins namespaces constructed as hji\plugin-name

        if (isset($parts[0]) && $parts[0] == 'hji' && isset($parts[1]) && !empty($parts[1]))
        {
            $serviceContainerClassName = "\\hji\\{$parts[1]}\\ServiceContainer";

            if (class_exists($serviceContainerClassName))
            {
                return $this->di()->get($serviceContainerClassName);
            }
        }

        // Fall back onto membershipServices

        return $this->membershipServices();
    }


    public function getMembershipServices()
    {
        _deprecated_function(__FUNCTION__, '2.2.2', $replacement='Component::membershipServices()');

        return parent::getServiceContainer();
    }


    public function getServices()
    {
        _deprecated_function(__FUNCTION__, '2.2.2', $replacement='Component::services()');

        return $this->services();
    }


    function getProjectRoute($projectNamespace = null)
    {
        if (!$route = $this->services()->getRouter()->getRoute())
        {
            return;
        }

        // Get project namespace if one is not provided

        if (!$projectNamespace)
        {
            $trace = debug_backtrace();

            if (isset($trace[1]['class']))
            {
                $projectNamespace = $trace[1]['class'];
            }
            else
            {
                trigger_error('Project Namespace can not be detected.', E_USER_ERROR);
            }
        }

        // Determine base namespace of the project i.e. hji\projectName

        $namespace = trim($projectNamespace, '\\');
        $parts = explode('\\', $namespace);
        $baseNamespace = $parts[0] . '\\' . $parts[1] . '\\';

        // Check if base namespace is part of the route

        $route = $this->services()->getRouter()->getRoute();

        if (substr($route->controllerClassName, 0, strlen($baseNamespace)) === $baseNamespace)
        {
            return $route;
        }

        return;
    }
}