<?php

namespace hji\membership\extension\postTypes;


use hji\common\postType\CustomPostType;
use hji\common\postType\HasAdminColumns;
use hji\common\postType\HasAdminFilters;
use hji\membership\extension\abstracts\Component;
use hji\membership\extension\taxonomies\TaxonomyManager;


class Factory extends Component
{
    public function createTaxonomies(array $taxonomySchema) : array
    {
        $taxonomyManager = $this->createTaxonomyManager();
        $taxonomies = [];

        foreach ($taxonomySchema as $taxonomy => $schema)
        {
            $taxonomies[] = $taxonomyManager->createTaxonomy($taxonomy, $schema);
        }

        return $taxonomies;
    }


    public function createPostTypeMeta(PostTypeDefinition $postType)
    {
    	$fieldsSchema = $postType->fieldsSchema();

    	foreach ($fieldsSchema as &$metabox)
	    {
		    $metabox['object_types'] = $metabox['object_types'] ?? [$postType->name];
	    }

        return $this->di()->create(PostTypeMeta::class, [
	        $fieldsSchema,
            $postType->fieldPrefix(),
            $postType->textdomain()]);
    }


    public function resolvePostType($classReference)
    {
        $customPostType = $this->di()->create($classReference);
        return new PostTypeModel($customPostType, $this);
    }


    public function createTaxonomyManager()
    {
        return new TaxonomyManager();
    }


    public function createPostTypeToRest()
    {
        return new PostTypeToREST();
    }


    public function createAdminCols(PostTypeDefinition $postType)
    {

        return $this->di()->create(PostTypeAdminCols::class, [
            $postType->adminColsSchema(),
            $postType->fieldPrefix(),
            $postType->textdomain()]);
    }


    public function createAdminFilters(PostTypeDefinition $postType)
    {

        return $this->di()->create(PostTypeAdminFilters::class, [
            $postType->adminFiltersSchema(),
            $postType->fieldPrefix(),
            $postType->textdomain()]);
    }


    public function createPostTypeDefinition(string $postType, array $args = [])
    {
        return $this->di()->create(PostTypeDefinition::class, [$postType, $args]);
    }
}