<?php

namespace hji\membership\extension\postTypes;


use hji\common\utils\StringUtil;


class PostTypeMeta
{
    protected $prefix;
    protected $textdomain;
    protected $schema = [];

    protected $fields = [];
    protected $metaboxes = [];

    /**
     * PostTypeMeta constructor.
     *
     * @param array       $fieldsSchema CustomPostType::fieldsSchema()
     * @param string|null $fieldPrefix
     * @param string|null $textdomain
     * @param StringUtil  $stringUtil
     */
    public function __construct(array $fieldsSchema, string $fieldPrefix = null, string $textdomain = null)
    {
        $this->prefix     = $fieldPrefix ?? '';
        $this->textdomain = $textdomain;
        $this->setProperties($fieldsSchema);
    }


    public function getFields()
    {
        return $this->fields;
    }


    public function getField(string $id)
    {
        $fields  = $this->getFields();
        $fieldId = $this->applyFieldPrefix($id);

        return $fields[$fieldId] ?? null;
    }


    public function getMetaboxes()
    {
        return $this->metaboxes;
    }


    public function applyFieldPrefix(string $fieldId)
    {
        return StringUtil::applyPrefix($fieldId, $this->prefix);
    }


    public function removeFieldPrefix(string $fieldId)
    {
        return StringUtil::removePrefix($fieldId, $this->prefix);
    }


    protected function setProperties(array $schema)
    {
        foreach ($schema as &$metabox)
        {
            $this->validateMetabox($metabox);

            $metabox = $this->localizeMetabox($metabox, $this->textdomain);

            foreach ($metabox['fields'] as &$field)
            {
                $this->validateField($field);

                $field['id'] = $this->applyFieldPrefix($field['id']);
                $this->metaboxes[$metabox['id']] = $metabox;
                $this->fields[$field['id']] = $field;
            }
        }

        $this->schema = $schema;
    }


    protected function localizeMetabox($metabox, $textdomain)
    {
        if (isset($metabox['title']))
            $metabox['title'] =  __($metabox['title'], $textdomain);

        if (!empty($metabox['fields']))
        {
            foreach ($metabox['fields'] as &$field)
            {
                $field = $this->localizeField($field, $textdomain);
            }
        }

        return $metabox;
    }


    protected function localizeField($field, $textdomain)
    {
        if (isset($field['desc']))
            $field['desc'] = __($field['desc'], $textdomain);

        if (isset($field['title']))
            $field['title'] = __($field['title'], $textdomain);

        return $field;
    }


    protected function validateMetabox($metabox)
    {
        $required = ['id', 'fields', 'object_types'];

        foreach ($required as $property)
        {
            if (!isset($metabox[$property]))
                throw new \Exception("Metabox schema is invalid. " .
                    "Missing one ore more properties: {$property}.");
        }
    }


    protected function validateField($field)
    {
        $required = ['id', 'type'];

        foreach ($required as $property)
        {
            if (!isset($field[$property]))
                throw new \Exception("Field schema is invalid. " .
                    "Missing one ore more properties: {$property}.");
        }
    }
}