<?php

namespace hji\membership\extension\postTypes;


use hji\common\postType\CustomPostType;


class PostTypeModel
{
    /** PostTypeDefinition::class */
    protected $postTypeDefinition;

    protected $postTypeProperties = [];

    protected $factory;

    protected $meta;

    protected $taxonomies;

    protected $adminCols;

    protected $adminFilters;


    public function __construct(CustomPostType $customPostType, Factory $factory)
    {
        $this->validate($customPostType);
        $this->factory = $factory;
        $this->postTypeDefinition = $factory->createPostTypeDefinition(
            $customPostType->post_type, get_object_vars($customPostType));
    }


    /**
     * @param string $property
     *
     * @return mixed|null
     */
    public function __get(string $property)
    {
        $properties = $this->getProperties();
        return $properties[$property] ?? null;
    }


    /**
     * @return CustomPostType
     */
    public function getPostTypeDefinition()
    {
        return $this->postTypeDefinition;
    }


    /**
     * @return array
     */
    public function getProperties()
    {
        if (!$this->postTypeProperties)
        {
            $properties = get_object_vars($this->getPostTypeDefinition());
            $properties['admin_cols'] = $this->getAdminCols();
            $properties['admin_filters'] = $this->getAdminFilters();

            $this->postTypeProperties = $properties;
        }

        return $this->postTypeProperties;
    }


    /**
     * @return array|null
     */
    public function getTaxonomies()
    {
        if (!$this->postTypeDefinition->taxonomiesSchema())
            return null;

        if (!$this->taxonomies)
            $this->taxonomies = $this->factory->createTaxonomies(
                $this->postTypeDefinition->taxonomiesSchema());

        return $this->taxonomies;
    }


    /**
     * @return array
     */
    public function getFields()
    {
        return $this->meta()->getFields();
    }


    /**
     * @param string $id
     *
     * @return mixed|null
     */
    public function getField($id)
    {
        return $this->meta()->getField($id);
    }


    /**
     * @return array
     */
    public function getMetaboxes()
    {
        return $this->meta()->getMetaboxes();
    }


    public function getAdminCols()
    {
        if (!$this->postTypeDefinition->adminColsSchema())
            return null;

        return $this->adminCols()->getColumns();
    }


    public function getAdminFilters()
    {
        if (!$this->postTypeDefinition->adminFiltersSchema())
            return null;

        return $this->adminFilters()->getFilters();
    }


    /**
     * @param string $fieldId
     *
     * @return string
     */
    public function applyFieldPrefix($fieldId)
    {
        return $this->meta()->applyFieldPrefix($fieldId);
    }


    /**
     * @param string $fieldId
     *
     * @return string
     */
    public function removeFieldPrefix($fieldId)
    {
        return $this->meta()->removeFieldPrefix($fieldId);
    }


    /**
     * @return PostTypeMeta
     */
    protected function meta()
    {
        if (!$this->meta)
            $this->meta = $this->factory->createPostTypeMeta($this->postTypeDefinition);

        return $this->meta;
    }


    protected function adminCols()
    {
        if (!$this->adminCols)
            $this->adminCols = $this->factory->createAdminCols($this->postTypeDefinition);

        return $this->adminCols;
    }


    protected function adminFilters()
    {
        if (!$this->adminFilters)
            $this->adminFilters = $this->factory->createAdminFilters($this->postTypeDefinition);

        return $this->adminFilters;
    }


    private function validate(CustomPostType $postType)
    {
        if (!$postType->post_type)
        {
            throw new \Exception('Custom Post Type ' .
                get_class($postType) . ' is missing required property: "post_type".');
        }
    }
}