<?php

namespace hji\membership\interfaces;

use hji\membership\Membership;
use hji\membership\models\ProductDependencies;
use hji\membership\models\WpCatalog;


/**
 * Class Product
 *
 * @since 2.0
 * @package hji\membership\interfaces
 */
abstract class Product
{
    const PACKAGE = 'hji-membership';

    /**
     * Main product file
     *
     * @var
     */
    protected $file;

    protected $slug;

    public $dependencies;

    /**
     * Product type
     * @var string - plugin|theme
     */
    protected $type = null;


    public function __construct($file)
    {
        $this->file = $file;
        $this->slug = basename(dirname($file));
        $this->dependencies = new ProductDependencies($this);
    }


    /**
     * @codeCoverageIgnore
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }


    /**
     * Returns full path to the main product file
     *
     * @codeCoverageIgnore
     * @return mixed
     */
    public function getFile()
    {
        return $this->file;
    }


    /**
     * @codeCoverageIgnore
     * @return string
     */
    public function getSlug()
    {
        return $this->slug;
    }

    abstract public function getData();


    /**
     * Determines if passed product array belongs to our package.
     * Intended for use on non-registered with membership products.
     *
     * @param $dataArray
     * @return bool
     */
    public static function isPackage($dataArray)
    {
        $headers = ProductDependencies::getFileHeadersHashmap();

        if (isset($dataArray[$headers[ProductDependencies::KEY_PACKAGE]]) && $dataArray[$headers[ProductDependencies::KEY_PACKAGE]] == self::PACKAGE)
        {
            return true;
        }

        return false;
    }


    function getUpdateInfo()
    {
        // Instead of calling each product info() separately - get its data from cached catalog
        $catalog = WpCatalog::browse();

        $t = $this->type . 's';

        if (!isset($catalog[$t]) || empty($catalog[$t]))
        {
            return;
        }

        foreach ($catalog[$t] as $product)
        {
            if ($product['slug'] == $this->slug)
            {
                // TODO: delete this
                $product = apply_filters('wp_catalog_data_filter', $product, $this->slug);
                return $product;
            }
        }
    }


    /**
     * @codeCoverageIgnore
     * @return null
     */
    public function getName()
    {
        $data = $this->getData();

        if (isset($data['Name']))
        {
            return $data['Name'];
        }

        return null;
    }
}