<?php

namespace hji\membership\models;

use hji\common\AbstractSettingsModel;
use hji\membership\Membership;


class Settings extends \hji\common\interfaces\AbstractSettingsModel
{
    protected static $__CLASS__ = __CLASS__; // Provide this in each singleton class.

    /** @var string     Options handle key */
    const KEY = 'hji-membership-settings';
    const GOOGLE_OPTIONS_KEY = 'hji-google-api';
    const DEFAULT_LOGO_HEIGHT = 75;

    /** @var string     Slipstream API License Key */
    public $licenseKey;

    /**
     * @since 2.6.0
     * @see https://slipstream.homejunction.com/docs/ws/areas#bias
     * @var string      Area Bias: postal-city, census-pace, none.
     */
    public $areaBias;

    /** @var array      Customer Information **/
    public $customer;


    public function __construct()
    {
        parent::__construct('membership', self::KEY);

        $this->loadOptions();

        $data = $this->settings;

        $this->licenseKey = !empty($data['license-key']) ? trim($data['license-key']) : null;
        $this->areaBias   = (isset($data['areaBias']) && !empty($data['areaBias'])) ? $data['areaBias'] : null;

        $this->customer['name']  =  !empty($data['customer']['name']) ? trim($data['customer']['name']) : null;
        $this->customer['email'] =  !empty($data['customer']['email']) ? trim($data['customer']['email']) : null;
        $this->customer['officeName']  =  !empty($data['customer']['officeName']) ? trim($data['customer']['officeName']) : null;
        $this->customer['officePhone'] =  !empty($data['customer']['officePhone']) ? trim($data['customer']['officePhone']) : null;
        $this->customer['mobilePhone'] =  !empty($data['customer']['mobilePhone']) ? trim($data['customer']['mobilePhone']) : null;
        $this->customer['fax'] =  !empty($data['customer']['fax']) ? trim($data['customer']['fax']) : null;
        $this->customer['address']['deliveryLine'] =  !empty($data['customer']['address']['deliveryLine']) ? trim($data['customer']['address']['deliveryLine']) : null;
        $this->customer['address']['city']  =  !empty($data['customer']['address']['city']) ? trim($data['customer']['address']['city']) : null;
        $this->customer['address']['state'] =  !empty($data['customer']['address']['state']) ? trim($data['customer']['address']['state']) : null;
        $this->customer['address']['zip']   =  !empty($data['customer']['address']['zip']) ? trim($data['customer']['address']['zip']) : null;
        $this->customer['photo'] = !empty($data['customer']['photo']) ? trim($data['customer']['photo']) : null;
        $this->customer['logo']  = !empty($data['customer']['logo']) ? trim($data['customer']['logo']) : null;
        $this->customer['logoHeight'] =
            !empty($data['customer']['logoHeight']) ? $data['customer']['logoHeight'] : self::DEFAULT_LOGO_HEIGHT
        ;

        $this->crm = $this->getCRMOptions();
        $this->googleApi = $this->getGoogleApiSettings();
    }


    /**
     * Sets Area Bias option to Census Places
     *
     * @since 2.6.0
     * @see docs/Area Bias.md
     *
     * @return $this
     */
    public function setCensusPlaceBias()
    {
        $censusPlacesBias = 'census-place';
        $this->overwriteAreaBias($censusPlacesBias);
        return $this;
    }


    /**
     * Sets Area Bias option to Postal City
     *
     * @since 2.6.0
     * @see docs/Area Bias.md
     *
     * @return $this
     */
    public function setPostalCityBias()
    {
        $postalCityBias = 'postal-city';
        $this->overwriteAreaBias($postalCityBias);
        return $this;
    }


    /**
     * Overrides area bias in the options
     *
     * @param $bias string  Area Bias
     *
     * @return $this
     */
    protected function overwriteAreaBias($bias)
    {
        if ($this->areaBias !== $bias)
        {
            $this->settings['areaBias'] = $bias;
            $this->setOptions($this->settings);
            $this->loadOptions();
        }
        return $this;
    }


    /**
     * Returns IRE CRM options
     *
     * Currently we only have the URL.
     *
     * @return array|bool CRM options array or false
     */
    protected function getCRMOptions()
    {
        $crmOptions = get_option('ire_crm');

        return (isset($crmOptions['url']) && !empty($crmOptions['url'])) ? $crmOptions : null;
    }


    protected function getGoogleApiSettings()
    {
        return new GoogleApiSettings();
    }


    public static function deleteOptions()
    {
        delete_option(self::KEY);
        delete_option('ire_crm');
    }
}
