<?php
/**
 * Action tracker for anonymous usage, until user registers
 *
 * @since 2.2.0
 */
namespace hji\membership\models;


use hji\common\utils\WPHelper;
use hji\membership\entities\Event;
use hji\membership\entities\UserActionEvent;
use hji\membership\interfaces\CRUD;
use hji\membership\interfaces\Loggable;


class UserActionLogger implements Loggable
{
    protected static $sessionKey = 'userActions';

    /**
     * @var bool Indicates that only anonymous usage should be logged
     */
    protected $anonymousUsage = true;


    /**
     * Recording/Reading Service
     * @var
     */
    protected $crudTransport;


    function __construct()
    {
//        register_shutdown_function(array($this, 'dumpData'));
    }

    function dumpData()
    {
        if (defined('DOING_AJAX') && DOING_AJAX)
        {
            return;
        }

        print '<pre>';
        print_r($this->getLog());
        print '</pre>';
        echo $this->getContent();
    }

    function setAnonymousLogger($bool)
    {
        $this->anonymousUsage = $bool;
        return $this;
    }


    function setCrudTransport(CRUD $transport)
    {
        $this->crudTransport = $transport;
        return $this;
    }


    /**
     * Records action and action data
     *
     * @param $event
     * @return $this
     */
    function log($event)
    {
        // Tracker only tracks Event objects

        if ($event instanceof UserActionEvent)
        {
            if ($this->anonymousUsage && $this->_isAnonymous())
            {
                $eventProperties = get_object_vars($event);
                $eventStdObj = new \stdClass();

                foreach ($eventProperties as $k => $v)
                {
                    $eventStdObj->$k = $v;
                }

                $eventStdObj->url        = $this->_getRequestingUrl();
                $eventStdObj->timestamp  = time();

                $this->saveEvent($eventStdObj);
            }
        }

        return $this;
    }


    function logUserAction($action, $attributes = null)
    {
        $event = new Event($action, $attributes);
        $this->log($event);
    }


    /**
     * Saves action and action data into user session
     *
     * @param $eventStdObj
     */
    private function saveEvent($eventStdObj)
    {
        $sessionData = $this->getLog();

        $sessionData[] = $eventStdObj;

        UserSession::set(self::$sessionKey, $sessionData);
    }


    /**
     * Returns tracked actions
     *
     * @return mixed
     */
    function getLog()
    {
        return UserSession::get(self::$sessionKey);
    }


    /**
     * @codeCoverageIgnore
     * @return string|void
     */
    function _getRequestingUrl()
    {
        if (defined( 'DOING_AJAX' ) && DOING_AJAX)
        {
            return $_SERVER['HTTP_REFERER'];
        }
        else
        {
            global $wp;

            return home_url($wp->request);
        }
    }


    function _isAnonymous()
    {
        if (!UserSession::getUser())
        {
            return true;
        }

        return false;
    }

}