<?php
namespace hji\membership\models;

use hji\membership\Membership;
use hji\membership\utils\Environment;
use WP_Error;

class WpCatalog
{
    const PROD_HOST = 'http://membership.homejunction.com';
    const STAGING_HOST  = 'http://membership-test.homejunction.com';


    private static function _getToken()
    {
        $membership = Membership::getInstance();

        return $membership->customerModel->license->token;
    }


    static function getHost()
    {
        $membership = \hji\membership\Membership::getInstance();

        if (Environment::is(Environment::DEV))
        {
            return self::STAGING_HOST;
        }

        return self::PROD_HOST;
    }


    private static function request($endpoint, $slug = false)
    {
        if (!$token = self::_getToken())
        {
            return false;
        }

        // Break token into parts to use last part for cacheToken

        $tParts = explode('-', $token);

        // Cache token keeps cache relevant to the actual authentication token

        $cacheToken = array_pop($tParts);

        $cacheTransport = new \hji\common\utils\WPCache();
        $cacheTransient = 'wpcatalog_' . $cacheToken. '_' . $endpoint . '_' . $slug;

        // Check if cache isset. In cache can be empty string or 0
        if (false !== ($cache = $cacheTransport->getCache($cacheTransient)))
        {
            return $cache;
        }

        $url = self::getUrl($endpoint, $slug);

        $result = wp_remote_get($url);

        // Check if connection response error, set to cache empty for 15 minutes
        // Example: Request: http://membership.homejunction.com/HJI-1060132C-BF65-4669-B22D-4AA796DA6561/browse
        // Response: AWAYS REYURN after 5 second: cURL error 28: Connection timed out after 5000 milliseconds
        if ($result instanceof WP_Error) {
            $cacheTransport->setCache($cacheTransient, null, 0.15);

            return false;
        }

        $json = wp_remote_retrieve_body($result);

        $data = json_decode($json, true);

        // Browse request returns data within ['result'] array, while the rest of requests do not.
        if ($endpoint === 'browse' && isset($data['result']))
        {
            $data = $data['result'];
        }

        if (JSON_ERROR_NONE == json_last_error() && $data)
        {
            $cacheTransport->setCache($cacheTransient, $data, 3);

            return $data;
        }

        return false;
    }


    static function browse()
    {
        return self::request('browse');
    }


    static function info($slug)
    {
        return self::request('info', $slug);
    }


    static function getInfoUrl($slug)
    {
        return self::getUrl('info', $slug);
    }


    static function getDownloadUrl($slug)
    {
        return self::getUrl('download', $slug);
    }


    static function getUrl($endpoint, $slug = false)
    {
        if (!$token = self::_getToken())
        {
            return false;
        }

        $url = self::getHost() . '/' . $token . '/' . $endpoint;

        $url .= ($slug) ? '/' . $slug : '';

        return $url;
    }


    static function download($slug)
    {
        return self::request('download', $slug);
    }
}
