<?php

namespace hji\membership\models\compliances;

use hji\common\utils\Views;
use hji\membership\models\data\Address;
//use hji\ResponsiveIDX\ResponsiveIDX;

class NwmlsCompliance
{

    private $urlSlug = 'nwmls-compliance';

    public function __construct()
    {
        $this->init();
    }

    /**
     * initialize the class and run our hooks
     * @codeCoverageIgnore
     */
    public function init()
    {
        $this->initActions();
        $this->initFilters();
    }

    /**
     * init wordpress actions
     */
    public function initActions()
    {
//        add_action('after_setup_theme', [$this, 'setupRoute']);
//        add_action('parse_request', [$this, 'hookRequest'], PHP_INT_MAX);
//        add_action('wp_footer', [$this, 'showComplianceCopyright']);
    }

    public function initFilters()
    {
//        add_filter('MapSearch/afterDisclaimer', [$this, 'generateComplianceCopyrightHtml']);
    }


    /**
     * Route Callback
     */
    public function setupRoute()
    {
        $router = $this->services()->getRouter();

        $router->add(
            NwmlsCompliance::class,
            NwmlsCompliance::class,
            NwmlsCompliance::class,
            [
                "^nwmls-compliance/?$",
                'index.php?hji-controller=NwmlsCompliance&&hji-action=render'
            ]
        );

        $router->mount();
    }


    /**
     * Request Callback
     * Alters the query and resulting post array.
     *
     * @param $query
     */
    public function hookRequest(&$query)
    {
        if (is_admin()) {
            return;
        }

        global $wp, $wp_query;

        if (( strtolower( $wp->request ) === $this->urlSlug ))
        {
            add_action('pre_get_posts', [$this, 'hookQuery'], PHP_INT_MAX);
            add_filter('the_posts', [$this, 'hookResults'], 10, 2);
        }
    }


    /**
     * Query Callback
     *
     * @param $query
     */
    public function hookQuery($query)
    {
        if ($query->is_main_query())
        {
            $query->set('post_type', 'page');
            $query->set('posts_per_page', 1);
            $query->set('name', 'nwmls-compliance-notice');
            $query->set('pagename', 'nwmls-compliance-notice');
            $query->max_num_pages = 0;
            $query->is_single = 1;
            $query->is_singular = 1;
            $query->is_page = 1;
            $query->is_home = false;
            $query->found_posts = 0;
        }
    }


    /**
     * Post Results Callback
     *
     * @param $posts
     * @param $query
     * @return array
     */
    public function hookResults($posts, $query)
    {
        if ($query->is_main_query())
        {
            remove_filter('the_content', 'wpautop');
            remove_filter('the_excerpt', 'wpautop');

            $posts = [(object) [
                "ID"                => time(),
                "comment_count"     => 0,
                "comment_status"    => 'closed',
                "ping_status"       => 'closed',
                "post_author"       => 1,
                "post_content"      => $this->renderContent(),
                "post_date"         => date("c"),
                "post_date_gmt"     => gmdate("c"),
                "post_modified"     => date("c"),
                "post_modified_gmt" => gmdate("c"),
                "post_excerpt"      => 'DMCA notice for NWMLS market compliance.',
                "post_name"         => 'nwmls-compliance-notice',
                "post_status"       => 'publish',
                "post_title"        => 'NWMLS DMCA Notice',
                "post_type"         => 'page',
            ]];
        }

        return $posts;
    }


    /**
     * Generates content for the page
     *
     * @return bool|string
     */
    protected function renderContent()
    {
        $context = get_option('hji-boulevard-settings');
        $address = $context['agent_address'];

        if (isset($address['street']) && $address['street'] !== '')
        {
            $address['deliveryLine'] = $address['street'];
        }

        $context['agent_address'] = new Address($address);
        $view = new Views(ResponsiveIDX::$dir . '/templates/');

        return $view->render('nwmls-dmca.twig', $context);
    }


    /**
     * Adds the NWMLS compliance page link to the site footer
     *
     * @param $copyright
     * @return string
     */
    public function generateComplianceCopyrightHtml($copyright = '')
    {
        // build dcma notice link for adding to the footer
        $dmca_page = site_url($this->urlSlug);
        $dcma_link = sprintf('<a href="%s" title="DMCA Notice" target="_blank">DMCA Notice</a>', $dmca_page);
        $copyright = $copyright . ' ' . $dcma_link;

        return $copyright;
    }

    public function showComplianceCopyright()
    {
        echo '<div class="market-compliance container text-center"><div class="row"><div class="col-md-12"></div> '
            . $this->generateComplianceCopyrightHtml()
            . '</div></div></div>';
    }
}
