<?php
namespace hji\membership\models\data;


class Images
{
    protected $collection = array();
    protected $primary;


    public function __construct(array $images = array())
    {
        if (!empty($images))
        {
            $this->addImages((array)$images);
        }
    }


    function addImages($images)
    {
        foreach ($images as $image)
        {
            $this->addImage($image);
        }
    }


    function addImage($image, $primary = false)
    {
        $imageObject = null;

        if ($image instanceof Image)
        {
            $imageObject = $image;
        }
        else if (is_array($image) || is_object($image))
        {
            $image = (array)$image;

            if (isset($image['url']))
            {
                $imageObject = new Image($image);
            }
            else if ($this->isValidUrl($image[0]))
            {
                $imageObject = new Image(array('url' => $image[0]));
            }
        }
        else if ($this->isValidUrl($image))
        {
            $imageObject = new Image(array('url' => $image));
        }

        if ($imageObject)
        {
            if ($primary)
            {
                $this->setPrimary($imageObject);
            }
            else
            {
                $this->collection[] = $imageObject;
            }
        }
    }


    function setPrimary(Image $image)
    {
        $this->primary = $image;
    }


    function getPrimary($defaultIndex = 0)
    {
        if ($this->primary)
        {
            return $this->primary;
        }
        else if ($defaultIndex !== false && isset($this->collection[$defaultIndex]))
        {
            return $this->collection[$defaultIndex];
        }
    }


    function getAll()
    {
        return $this->collection;
    }


    /**
     * @param $url image url to validate
     *
     * Slipstream API may return URLs without protocol as in //url
     *                   therefore we can't use filter_var($image[0], FILTER_VALIDATE_URL)
     *                   since it invalidates short hand urls
     * @return bool
     */
    protected function isValidUrl(string $url)
    {
        $hosts = ['http://', 'https://', '//'];

        foreach ($hosts as $host) {
            if (substr($url, 0, strlen($host)) === $host)
            {
                return true;
            }
        }
        return false;
    }
}