<?php
namespace hji\membership\models\webservice\abstracts;


use hji\membership\abstracts\Model;
use hji\membership\models\webservice\interfaces\ISearchable;
use hji\membership\models\webservice\interfaces\IGettable;
use hji\membership\models\webservice\WebServiceOptions;


abstract class WebService extends Model implements ISearchable, IGettable
{
    public function search(){}
    public function get(){}

    protected $path;

    protected $rawParameters;
    protected $filters;
    protected $options;
    protected $method;


    public function __construct(WebServiceFilters $filters, WebServiceOptions $options)
    {
        $this->filters = $filters;
        $this->options = $options;
    }


    public function getData()
    {
        return $this->call();
    }


    public function setParameters(array $parameters)
    {
        $this->rawParameters = $parameters;

        if ($this->filters && $this->options)
        {
            if (!empty($parameters))
            {
                foreach ($parameters as $k => $v)
                {
                    $this->filters->set($k, $v);
                    $this->options->set($k, $v);
                }
            }
        }
    }


    function getParameters()
    {
        $params = array();

        if ($this->filters)
        {
            $params = array_merge($params, $this->filters->getAll());
        }

        if ($this->options)
        {
            $params = array_merge($params, $this->options->getAll());
        }

        return array_filter($params);
    }


    public function setMethod($method)
    {
        $this->method = $method;
    }


    public function call()
    {
        if (is_callable(array($this, $this->method)))
        {
            return call_user_func(array($this, $this->method));
        }
        else
        {
            throw new \Exception("WebService method {$this->method} does not exist.");
        }
    }


    public function getPath()
    {
        $path = $this->path;

        if ($this->method)
        {
            $path .= '/' . $this->method;
        }

        return $path;
    }


    public function getFilters()
    {
        return $this->filters;
    }


    public function getOptions()
    {
        return $this->options;
    }
}