<?php
namespace hji\membership\services;

use Dice\Dice;
use Dice\Rule;
use Dice\Instance;


class DependencyInjectorInterface
{
    private static $DI;

    private function getDi()
    {
        if (!self::$DI)
        {
            self::$DI = new Dice();
        }

        return self::$DI;
    }


    /**
     * Creates new instance of an object
     *
     * @param       $component
     * @param array $args
     * @param null  $callback
     * @param bool  $forceNewInstance
     * @return object
     * @throws \Exception
     */
    public function create($component, array $args = array(), $callback = null, $forceNewInstance = false)
    {
        $di = $this->getDi();

        return $di->create($component, $args, $callback, $forceNewInstance);
    }


    /**
     * Returns an instance of an object
     * Singleton.
     */
    public function get($component, array $args = array(), $callback = null, $forceNewInstance = false)
    {
        $di = $this->getDi();

        // Check if component already has a rule
        if ($component instanceof Instance) $component = $component->name;
        $component = trim($component, '\\');

        // Since getRule returns a new rule if there's no existing one
        // we should detect if it's just a blank rule
        $rule = $di->getRule($component);
        $_blankRule = $this->createRule();

        if ($rule == $_blankRule)
        {
            $rule->shared = true;
            $di->addRule($component, $rule);
        }

        return $di->create($component, $args, $callback, $forceNewInstance);
    }


    /**
     * Creates new rule for dependency injection
     *
     * @return Rule
     */
    public function createRule()
    {
        return new Rule();
    }


    public function addRule($name, Rule $rule)
    {
        $di = $this->getDi();
        $di->addRule($name, $rule);
    }


    public function instance($name)
    {
        return new Instance($name);
    }
}