<?php
namespace hji\membership\services;


use hji\membership\Membership;

class ServiceContainer extends DependencyInjector
{
    /**
     * Returns a current instance of models\Customer
     *
     * @since 2.2.0
     * @return mixed
     */
    function getCustomer()
    {
        $service = 'hji\\membership\\models\\Customer';

        return $this->di()->get($service);
    }


    /**
     * Returns a current authorized instance of hji\common\utils\APIClient
     *
     * @since 2.2.0
     * @return mixed
     */
    function getApiClient()
    {
        $service = 'hji\\membership\\controllers\\APIClient';
        $apiController = $this->di()->get($service);

        return $apiController->api;
    }


    /**
     * Returns a current instance of controllers\Dispatcher
     *
     * @since 2.2.0
     * @return mixed
     */
    function getDispatcher()
    {
        return $this->di()->get('hji\\membership\\controllers\\Dispatcher');
    }


    /**
     * Returns a current instance of models\UserActionLogger
     *
     * @since 2.2.0
     * @return mixed
     */
    public function getUserActionLogger()
    {
        return $this->di()->get('hji\\membership\\models\\UserActionLogger');
    }


    /**
     * @since 2.2.0
     * @param $templatesPath
     * @return object
     */
    public function getView($templatesPaths)
    {
        $service = 'hji\\common\\utils\\Views';
        return $this->di()->create($service, array($templatesPaths));
    }


    /**
     * @since 2.2.2
     * @return object
     */
    public function getRequest()
    {
        return $this->di()->get('hji\\membership\\services\\Request');
    }


    /**
     * @since 2.2.2
     * @return object
     */
    public function getPaths()
    {
        return $this->di()->get('hji\\common\\utils\\Paths', array(Membership::$file));
    }


    /**
     * @since 2.2.2
     * @return object
     */
    public function getRouter()
    {
        return $this->di()->get('hji\\membership\\services\\Router');
    }


    public function getRegistry()
    {
        return $this->di()->get('hji\\membership\\services\\Registry');
    }
}