<?php
namespace hji\membership\services;


/**
 * Currently just an entry point for future developments
 *
 * @package hji\membership\services
 */
class SocialGraph
{
    function __construct()
    {

    }


    function init()
    {
        $this->setActions();
    }


    protected function setActions()
    {
        add_filter('wpseo_metadesc',        array($this, 'filterWpSeoMetaDescription'));
        add_filter('wpseo_opengraph_type',  array($this, 'filterOGtype'));
        add_action('wpseo_opengraph',       array($this, 'addOGplaceCoordinates'));
        add_action('wpseo_opengraph',       array($this, 'addOGimages'));
    }


    function filterWpSeoMetaDescription($metadesc)
    {
        global $post;

        if ($this->_isListingPost($post))
        {
            $meta = array();

            if ($post->listing->listPrice)
            {
                $listPrice = filter_var($post->listing->listPrice, FILTER_SANITIZE_NUMBER_INT);
                $meta[] = '$' . number_format($listPrice);
            }

            if ($post->listing->beds)
            {
                $meta[] = $post->listing->beds . '-beds';
            }

            if ($post->listing->baths_total)
            {
                $meta[] = $post->listing->baths_total . '-baths';
            }

            $metadesc = implode(' &#8226; ', $meta) . '.';

            if ($post->listing->description)
            {
                $metadesc .= ' ' . $post->listing->description;
            }
        }

        $metadesc = apply_filters('hji_socialgraph_metadesc', $metadesc, $post);

        return $metadesc;
    }


    public function filterOGtype($type)
    {
        global $post;

        if ($this->_isListingPost($post))
        {
            $type = 'place';
        }

        return $type;
    }


    public function addOGplaceCoordinates()
    {
        global $post;

        if ($this->_isListingPost($post) && isset($GLOBALS['wpseo_og']))
        {
            if ($post->listing->coordinates_latitude &&
                $post->listing->coordinates_longitude)
            {
                $GLOBALS['wpseo_og']->og_tag('place:location:latitude', $post->listing->coordinates_latitude);
                $GLOBALS['wpseo_og']->og_tag('place:location:longitude', $post->listing->coordinates_longitude);
            }
            else if ($post->listing->coordinates)
            {
                $GLOBALS['wpseo_og']->og_tag('place:location:latitude', $post->listing->coordinates->latitude);
                $GLOBALS['wpseo_og']->og_tag('place:location:longitude', $post->listing->coordinates->longitude);
            }
        }
    }


    protected function _isListingPost($post)
    {
        if ($post && is_main_query() &&
            isset($post->listing) && in_array($post->post_type, $this->_getSupportedPostTypes()))
        {
            return true;
        }

        return false;
    }


    // @TODO: we've added 'idx-listing' here because we're not always using a 'post' post-type for listings (think blvd)
    protected function _getSupportedPostTypes()
    {
        // @TODO: remove the 'post' from this array once a post type is fully established for rIDX listings
        $supportedTypes = array('post', 'idx-listing', 'nc-listing');
        $wpseo_opt      = $this->_getWPseoOptions();

        if (isset($wpseo_opt[ 'metadesc-nc-listing']) && !empty($wpseo_opt['metadesc-nc-listing']))
        {
            if ($key = array_search('nc-listing', $supportedTypes))
            {
                unset($supportedTypes[$key]);
            }
        }

        return $supportedTypes;
    }


    protected function _getWPseoOptions()
    {
        $options = array();

        if (class_exists('\WPSEO_Options'))
        {
            $options = \WPSEO_Options::get_all();
        }

        return $options;
    }


    /**
     * Adds images from listings to open graph meta tags
     */
    public function addOGimages()
    {
        global $post;

        if ($this->_isListingPost($post) && isset($GLOBALS['wpseo_og']))
        {
            if ($post->listing->images)
            {
                foreach ($post->listing->images->getAll() as $image)
                {
                    $GLOBALS['wpseo_og']->og_tag('og:image', esc_url($image->src). '?width=1000');
                }

                $GLOBALS['wpseo_og']->og_tag('og:image:width', '1000');
                $GLOBALS['wpseo_og']->og_tag('og:image:height', '750');

            }
        }
    }
}
