<?php
namespace hji\common;


use hji\common\plugin\PluginAPI;
use hji\membership\services\ServiceContainer;


abstract class Plugin
{
    public      $name;
    public      $slug;
    public      $file; // absolute path to the plugin's main file
    protected   $pluginAPI;

    /** @return mixed - Called by Membership when plugins loaded.*/
    abstract public function init();


    public function __construct($name, $slug, $mainFilePath, PluginAPI $pluginAPI = null)
    {
        $this->name = $name;
        $this->slug = $slug;
        $this->file = $mainFilePath;
        $this->pluginAPI = $pluginAPI;
    }


    public function api()
    {
        if (!$this->pluginAPI instanceof PluginAPI)
            $this->pluginAPI = $this->createAPI();

        return $this->pluginAPI;
    }


    /**
     * Runs on plugin activation
     * If the plugin App class contains a method called doOnPluginActivation() it will also be called.
     */
    public function doOnPluginActivation()
    {
        flush_rewrite_rules();
    }


    /**
     * Runs on plugin deactivation
     * If the plugin App class contains a method called doOnPluginDeactivation() it will also be called.
     */
    public function doOnPluginDeactivation()
    {
        flush_rewrite_rules();
    }


    protected function createAPI()
    {
        return new PluginAPI(new ServiceContainer());
    }

}