<?php
namespace hji\common\factories;


use hji\common\interfaces\WidgetShortcode;
use hji\common\models\mcePluginModel;


class WidgetTrioFactory
{
    /**
     * Widget's Class name with fully qualified namespace
     * @var
     */
    protected $widgetClass;

    /**
     * Widget Object
     * @var
     */
    public $widget;

    /**
     * Shortcode objcet
     * @var
     */
    public $shortcode;

    /**
     * TinyMCE Plugin Model
     * @var
     */
    public $mcePluginModel;


    /**
     * Shortcode name
     * @var null
     */
    protected $shortcodeName = null;

    /**
     * TinyMCE Plugin Button Icon
     * @var null
     */
    protected $iconUrl    = null;


    /**
     * Dimensions of the TinyMCE dialog window.
     * @var
     */
    protected $height;
    protected $width;


    function __construct($widgetClass)
    {
        $this->widgetClass = $widgetClass;
    }


    function setShortcodeName($name)
    {
        $this->shortcodeName = $name;
    }


    function setIconUrl($iconUrl)
    {
        $this->iconUrl = $iconUrl;
    }


    function setHeight($height)
    {
        $this->height = $height;
    }


    function setWidth($width)
    {
        $this->width = $width;
    }


    function create()
    {
        add_action('widgets_init', array($this, 'registerWidget'));
        add_action('widgets_init', array($this, 'registerOthers'));
    }


    /**
     * Returns widget object prior-registered with WP
     * This allows us to avoid instantiation of the widget class multiple times,
     * which can be pretty resource-heavy depending on implementation
     *
     * @return mixed
     */
    function getWidgetInstance()
    {
        global $wp_widget_factory;

        $widget_obj = $wp_widget_factory->widgets[$this->widgetClass];

        if ( !is_a($widget_obj, 'WP_Widget') )
            return;

        return $widget_obj;
    }


    /**
     * Registers Shortcode and TinyMCE Plugin with WP
     */
    function registerOthers()
    {
        // Get prior-registered widget instance
        if (!$this->widget = $this->getWidgetInstance())
        {
            return;
        }

        // Shortcode Object
        $this->shortcode    = new WidgetShortcode($this->widget, $this->shortcodeName);

        // MCE Plugin Model
        $this->mcePluginModel = $this->_createMcePluginModel($this->widget, $this->shortcode);



        $this->_registerShortcode();

        $this->_registerMcePlugin();
    }


    /**
     * Registers widget with WordPress
     */
    function registerWidget()
    {
        register_widget($this->widgetClass);
    }


    /**
     * Registers shortcode with WordPress
     */
    protected function _registerShortcode()
    {
        $this->shortcode->register();
    }


    protected function _registerMcePlugin()
    {
        $mcePluginFactory = new mcePluginFactory($this->mcePluginModel);
        $mcePluginFactory->create();
    }


    /**
     * Creates and modifies McePlugin Model for our Widget
     *
     * @param $widget
     * @param $shortcode
     * @return mcePluginModel
     */
    protected function _createMcePluginModel($widget, $shortcode)
    {
        $mcePluginModel = new mcePluginModel($widget->id_base, $widget->name);

        // Set MCE plugin's shortcode name

        $mcePluginModel->setShortcodeName($shortcode->name);

        // Set button icon

        if ($this->iconUrl)
        {
            $mcePluginModel->setIconUrl($this->iconUrl);
        }

        if ($this->height)
        {
            $mcePluginModel->setHeight($this->height);
        }

        if ($this->width)
        {
            $mcePluginModel->setWidth($this->width);
        }

        // Set content callback - which will be called only when tinyMCE Dialog is opened.

        $mcePluginModel->setCallback(array($this, 'getMceFormView'));

        return $mcePluginModel;
    }


    /**
     * Returns widget view to be used in MCE Plugin
     *
     * @return mixed
     */
    function getMceFormView()
    {
        $widget = clone $this->widget;
        // Tell widget object that it's inside MCE popup window
        $widget->isTinyMcePlugin = true;

        return $widget->getFormContent(array());
    }

}