<?php
namespace hji\common\factories;

use hji\common\models\PluginJsVars;
use hji\common\models\mcePluginModel;
use hji\membership\Membership;


class mcePluginFactory
{
    protected $mcePluginModel;


    function __construct(mcePluginModel $mcePluginModel)
    {
        $this->mcePluginModel = $mcePluginModel;
    }


    function create()
    {
        if (current_user_can('edit_posts') && current_user_can('edit_pages'))
        {
            $this->registerActionHooks();
        }
    }


    protected function registerActionHooks()
    {
        add_action('wp_ajax_' . $this->mcePluginModel->getAction(), array($this, 'renderMcePopupDialog'));

        if (get_user_option('rich_editing') == 'true')
        {
            add_action('admin_enqueue_scripts', array($this, 'loadScripts'));

            add_filter('mce_external_plugins', array($this, 'registerPlugin'));

            // Adding Home Junction MCE Buttons for grouping together
            add_filter('hji_mce_buttons', array($this, 'registerButton'));
        }
    }


    function loadScripts($hook)
    {
        if ($hook == 'post.php' || $hook == 'post-new.php')
        {
            // Load plugin model as JS var, so it can be ready by tinymcePluign.js registered via registerPlugin().

            $jsVars = new PluginJsVars('tinymcePlugins');
            $jsVars->setVar($this->mcePluginModel->getHandle(), $this->mcePluginModel->exportObject());
        }

        if ($this->isMcePopup())
        {
            wp_enqueue_script('tinymce_popup', get_option('siteurl') . '/wp-includes/js/tinymce/tiny_mce_popup.js');
            wp_enqueue_script('tinymce_form_utils', get_option('siteurl') . '/wp-includes/js/tinymce/utils/form_utils.js');
            wp_enqueue_script('hji_tinymce_shortcode', Membership::$url . '/resources/scripts/build/components/tinymceToShortcode.js');
        }
    }


    function isMcePopup()
    {
        if (isset($_REQUEST['action']) && $_REQUEST['action'] == $this->mcePluginModel->getAction())
        {
            return true;
        }

        return false;
    }


    function registerPlugin($plugin_array)
    {
        $plugin_array[$this->mcePluginModel->getHandle()] = Membership::$url . '/resources/scripts/build/components/tinymcePlugin.js';

        return $plugin_array;
    }


    function registerButton($buttons)
    {
        array_push($buttons, "|", $this->mcePluginModel->getHandle());

        return $buttons;
    }


    /**
     * Loads TinyMCE window for a widget via AJAX
     */
    function renderMcePopupDialog()
    {
        // check for rights

        if (!current_user_can('edit_pages') && !current_user_can('edit_posts'))
        {
            die(__("You are not allowed to be here"));
        }

        if (!$_REQUEST['plugin'])
        {
            die();
        }

        $view = new \hji\common\utils\Views(Membership::$dir . '/common/views/');

        $vars['mcePluginModel'] = $this->mcePluginModel;

        echo $view->render('tinymce-page', $vars);

        die();
    }

}