<?php
namespace hji\common\interfaces;


abstract class Shortcode
{
    /**
     * Shortcode name
     * @var
     */
    protected $name;

    /**
     * Parameters supported by the shortcode
     * @var
     */
    protected $parameters = array();


    /**
     * @param       $name
     * @param array $parameters
     */
    function __construct($name, $parameters = array())
    {
        $this->name         = str_replace('-', '_', $name);
        $this->parameters   = $parameters;
    }


    function __get($name)
    {
        if (property_exists($this, $name))
        {
            return $this->$name;
        }
    }


    function register()
    {
        if ($this->name)
        {
            add_shortcode($this->name, array($this, 'doShortcode'));
        }
    }


    /**
     * Processes shortcode atts and renders shortcode content
     *
     * @param $atts
     * @return mixed|string
     */
    function doShortcode($atts)
    {
        // Validate atts against supported parameters

        if ($this->parameters)
        {
            $params = $this->validateParameters($atts);

            if (is_wp_error($params))
            {
                // Render error only if user is logged in

                if (is_user_logged_in())
                {
                    return $params->get_error_message();
                }

                return;
            }
        }
        else
        {
            $params = $atts;
        }

        return $this->render($params);
    }


    /**
     * Renders shortcode's content
     *
     * @param array $params
     * @internal param $atts - attributes passed via shortcode.
     * @return mixed
     */
    abstract function render($params = array());


    /**
     * Validates shortcode attributes against supported parameters
     *
     * @param $atts
     * @return \WP_Error
     */
    function validateParameters($atts)
    {
        if (empty($atts))
        {
            return $atts;
        }

        $params = $atts;
        $invalid = false;

        foreach ($this->parameters as $param)
        {
            $supportedParams[strtolower($param)] = $param;
        }

        $supportedKeys = array_keys($supportedParams);

        foreach($atts as $k => $v)
        {
            $key = strtolower($k);

            if (is_int($k) || strpos($k, ' ') !== false || !in_array($key, $supportedKeys))
            {
                $invalid[] = $k;
            }
            else
            {
                // If attribute exists in the array of supported parameters
                // unset the lowercase attribute passed by shortcode
                unset($params[$k]);
                // and set it's corresponding camelCase key from the supported parameters array
                $params[$supportedParams[$key]] = $v;
            }
        }

        if ($invalid)
        {
            $message = sprintf('Shortcode contains invalid parameters: %s.', implode(', ', $invalid));

            return new \WP_Error('hji shortcode', $message);
        }

        return $params;
    }
} 