<?php

namespace hji\common\interfaces;

use \hji\common\utils\Views;


abstract class Widget extends \WP_Widget
{
    /**
     * Supported fields are currently not used within the Widget object,
     * but they are supported by Shortcodes class, which will validate
     * shortcode attributes against widget's supported fields, if they are defined.
     * [\hji\common\interfaces\Shortcode::validateParameters()]
     *
     * @var
     */
    public static $supportedFields;

    /**
     * TinyMCE flag that is being set when Widget object is created
     * within the tinyMCE Plugin's PopUp window.
     *
     * @var bool
     */
    public $isTinyMcePlugin = false;


    /**
     * Renders widget content on the page
     * Implementation of WP_Widget::widget();
     *
     * @param array $args
     * @param array $instance
     */
    function widget($args, $instance)
    {
        echo $this->getWidgetContent($args, $instance);
    }


    /**
     * Renders form content on the page
     * Implementation of WP_Widget::form();
     *
     * @param array $instance
     * @return string|void
     */
    function form($instance)
    {
        echo $this->getFormContent($instance);
    }


    /**
     * Returns widget content
     *
     * @param $instance
     * @param $args
     * @return bool|string
     */
    function getWidgetContent($args, $instance)
    {
        $vars['instance']   = $this->prepareWidget($args, $instance);
        $vars['args']       = $args;
        $vars['widget']     = $this;

        return $this->getWidgetView()->render(false, $vars);
    }


    /**
     * Returns form content
     *
     * @param $instance
     * @return bool|string
     */
    function getFormContent($instance)
    {
        $vars           = $this->prepareForm($instance);
        $vars['widget'] = $this;

        if (!isset($vars['instance']))
        {
            $vars['instance'] = $instance;
        }

        return $this->getFormView()->render(false, $vars);
    }


    function get_field_name($field_name)
    {
        if ($this->isTinyMcePlugin)
        {
            return $field_name;
        }

        return parent::get_field_name($field_name);
    }


    function get_field_id($field_id)
    {
        if ($this->isTinyMcePlugin)
        {
            return $field_id;
        }

        return parent::get_field_id($field_id);
    }


    /**
     * IMPORTANT: Assign .shortcode-value class to form input fields markup
     * which values should be part of the shortcode
     *
     * @return Views
     */
    function getFormView()
    {
        return $this->getView($this->getFormTemplateFile());
    }


    function getWidgetView()
    {
        return $this->getView($this->getWidgetTemplateFile());
    }


    function getView($filePath)
    {
        $path =  dirname($filePath);
        $templateName = str_replace('.phtml', '', basename($filePath));

        $view = new Views($path);
        $view->setFileName($templateName);

        return $view;
    }


    abstract function prepareWidget($instance, $args);

    abstract function prepareForm($instance);

    abstract function getWidgetTemplateFile();

    abstract function getFormTemplateFile();
}