<?php
namespace hji\common\models;


use hji\membership\Membership;

class mcePluginModel
{
    /**
     * Plugin handle - name with no spaces or hyphens
     * @var
     */
    protected $handle;

    /**
     * Plugin name - user readable name
     * @var
     */
    protected $name;
    protected $longName;

    /**
     * Action string passed via ajax
     *
     * @var string
     */
    protected $action;

    /**
     * Icon used for editor panel button
     * @var null
     */
    protected $iconUrl;

    protected $width    = 500;
    protected $height   = 300;


    /**
     * HTML content ready to be rendered
     * @var
     */
    protected $content;


    protected $getContentCallback;


    function __construct($handle, $name)
    {
        $this->handle   = str_replace('-', '_', $handle);
        $this->name     = $name;
        $this->iconUrl = Membership::$url . '/resources/images/wp-icon32.png';
        $this->action   = 'hji_' . $this->handle . '_tinymce';
    }


    function setAction($action)
    {
        $this->action = $action;
    }


    function getAction()
    {
        return $this->action;
    }


    function setLongName($longName)
    {
        $this->longName = $longName;
    }


    function getHandle()
    {
        return $this->handle;
    }


    function getName()
    {
        return $this->name;
    }


    function getLongName()
    {
        if (!$this->longName)
        {
            return $this->name;
        }

        return $this->longName;
    }


    function setIconUrl($iconUrl)
    {
        $this->iconUrl = $iconUrl;
    }


    function getIconUrl()
    {
        return $this->iconUrl;
    }


    /**
     * @return mixed
     */
    public function getHeight()
    {
        return $this->height;
    }


    /**
     * @param mixed $height
     */
    public function setHeight($height)
    {
        $this->height = $height;
    }


    /**
     * @return mixed
     */
    public function getWidth()
    {
        return $this->width;
    }


    /**
     * @param mixed $width
     */
    public function setWidth($width)
    {
        $this->width = $width;
    }


    function setShortcodeName($shortcodeName)
    {
        $this->shortcodeName = $shortcodeName;
    }


    function getShortcodeName()
    {
        if (!$this->shortcodeName)
        {
            return $this->handle;
        }

        return $this->shortcodeName;
    }


    function exportObject()
    {
        $obj = new \stdClass();

        /**
         * Get only protected properties, since we need them
         * to set TinyMCE widget. No recursive objects should be
         * included, otherwise we won't be able to encode them
         * into JSON.
         */

        $reflect = new \ReflectionClass($this);
        $props   = $reflect->getProperties(\ReflectionProperty::IS_PROTECTED);

        foreach ($props as $property)
        {
            $name = $property->getName();

            if ($name != 'getContentCallback')
            {
                $obj->$name = $this->$name;
            }
        }

        return $obj;
    }


    function setContent($content)
    {
        $this->content = $content;
    }


    function getContent()
    {
        if ($this->getContentCallback)
        {
            return call_user_func($this->getContentCallback);
        }

        return $this->content;
    }


    /**
     * Callback that will be called to return mcePlugin content.
     * Since the model is created earlier in the process,
     * we don't want it to do any heavy processing
     * until the plugin's window is actually opened.
     * This is when we'll trigger this callback.
     *
     * @param $callback
     */
    function setCallback($callback)
    {
        $this->getContentCallback = $callback;
    }
} 