<?php

namespace hji\common\utils;

use ArrayObject;

class ConfigObject extends ArrayObject
{

    public function __construct($config = [])
    {
        if (is_string($config))
        {
            $config = $this->loadConfigFromUri($config) ?: [];
        }
        parent::__construct($config, ArrayObject::ARRAY_AS_PROPS);
    }


    /**
     * Check whether the Config has a specific key.
     *
     * @param $key string The key to check for.
     * @return bool If the key exists
     */
    public function has_key($key)
    {
        return array_key_exists($key, (array) $this);
    }


    /**
     * Get the value of a specific key from the Config
     *
     * @param $key string The key to get the value for.
     * @return mixed Value of the config key
     */
    public function get_key($key)
    {
        return $this[$key];
    }


    /**
     * Get an array of all keys from the Config
     *
     * @return array Array of config keys
     */
    public function get_keys()
    {
        return array_keys((array) $this);
    }


    /**
     * Loads the contents of a Config by file reference
     *
     * @param $uri string URI of the Config resource
     * @return array|null Raw data loaded from the resource or null if no data found
     */
    protected function loadConfigFromUri($uri)
    {
        try {
            // Try to load the file through PHP's include().
            // Make sure we don't accidentally create output.
            ob_start();
            $data = include $uri;
            ob_end_clean();
            return (array) $data;
        }
        catch (\Exception $exception)
        {
            throw new \RuntimeException(
                sprintf('Could not include PHP config file "%1$s". Reason: "%2$s".'),
                $exception->getCode(),
                $exception
            );
        }
    }
}