<?php
namespace hji\common\utils;


use hji\membership\extension\abstracts\Component;


/**
 * Class Paths - SINGLETON
 *
 * USAGE:
 *
 * Invoke the class as a singleton via dependency injector:
 *
 *  $paths = $this->di()->get('hji\\common\\utils\\Paths');
 *  $paths->setPath('folder', 'path/to/handle/folder);
 *
 *  $folderPath = $paths->getPath('folder);
 *  $folderUrl = $paths->getUrl('folder);
 *
 *
 * To use in other plugins, simply extend the class
 * and pass $pathDirPath to the current plugin.
 * Invoke as indicated above.
 *
 * EXTENSION EXAMPLE:
 *
 *  class Paths extends \hji\common\utils\Paths
 *  {
 *      function __construct($pluginFile)
 *      {
 *          parent::__construct($pluginFile);
 *      }
 *  }
 *
 * @since 2.2.2
 * @package hji\common\utils
 */
class Paths extends Component
{
    private $slug;
    private $paths;


    /**
     * Absolute path to the plugin's file.
     *
     * @param $pluginFile
     */
    function __construct($pluginFile)
    {
        $this->slug = basename(dirname($pluginFile));
        $this->dir  = plugin_dir_path($pluginFile);
        $this->url  = plugin_dir_url($pluginFile);
    }


    function getSlug()
    {
        return $this->slug;
    }


    function setPath($key, $relativePath)
    {
        $this->paths[$key] = trim($relativePath ,'/');
    }


    function getPath($key)
    {
        if (isset($this->paths[$key]))
        {
            return $this->dir . $this->paths[$key];
        }
        else
        {
            trigger_error("Path for {$key} doesn't exist.", E_USER_ERROR);
        }
    }


    // TODO: maybe trailing slash all urls - it makes more sense this way
    function getUrl($key)
    {
        if (isset($this->paths[$key]))
        {
            return $this->url . $this->paths[$key];
        }
        else
        {
            trigger_error("Url for {$key} doesn't exist.", E_USER_ERROR);
        }
    }

}