<?php
/**
 * Created by PhpStorm.
 * User: maxchirkov
 * Date: 10/8/14
 * Time: 10:07 AM
 */

namespace hji\common\utils;

SiteInfo::getInstance();

class SiteInfo
{

    /**
     * @var    object    Instance of this class
     */
    private static $instance;

    /**
     * Class constructor
     */
    function __construct()
    {
        if (isset($_GET['qq']))
        {
            $this->init();
        }
    }


    function init()
    {
        $query = json_decode(base64_decode($_GET['qq']));

        if (json_last_error() != JSON_ERROR_NONE ||
            !isset($query->siteinfo))
        {
            return;
        }

        $filters = isset($query->filters) ? $query->filters : null;
        $info = json_encode($this->getSiteInfo($filters));
        die($info);
    }

    /**
     * Get the singleton instance of this class
     *
     * @return object
     */
    public static function getInstance() {
        if ( ! ( self::$instance instanceof self ) ) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    /**
     * Main tracking function.
     */
    function getSiteInfo($filters = array()) {

//        $transient_key = 'yoast_tracking_cache';
//        $data          = get_transient( $transient_key );
//
//        // bail if transient is set and valid
//        if ( $data !== false ) {
//            return;
//        }
//
//        // Make sure to only send tracking data once a week
//        set_transient( $transient_key, 1, WEEK_IN_SECONDS );

        // Start of Metrics
        global $blog_id, $wpdb;

//        $hash = get_option( 'Yoast_Tracking_Hash', false );
//
//        if ( ! $hash || empty( $hash ) ) {
//            // create and store hash
//            $hash = md5( site_url() );
//            update_option( 'Yoast_Tracking_Hash', $hash );
//        }

        $pts        = array();
        $post_types = get_post_types( array( 'public' => true ) );
        if ( is_array( $post_types ) && $post_types !== array() ) {
            foreach ( $post_types as $post_type ) {
                $count             = wp_count_posts( $post_type );
                $pts[ $post_type ] = $count->publish;
            }
        }
        unset( $post_types );

        $comments_count = wp_count_comments();

        $theme_data     = wp_get_theme();
        $theme          = array(
            'name'       => $theme_data->display( 'Name', false, false ),
            'theme_uri'  => $theme_data->display( 'ThemeURI', false, false ),
            'version'    => $theme_data->display( 'Version', false, false ),
            'author'     => $theme_data->display( 'Author', false, false ),
            'author_uri' => $theme_data->display( 'AuthorURI', false, false ),
        );
        $theme_template = $theme_data->get_template();
        if ( $theme_template !== '' && $theme_data->parent() ) {
            $theme['template'] = array(
                'version'    => $theme_data->parent()->display( 'Version', false, false ),
                'name'       => $theme_data->parent()->display( 'Name', false, false ),
                'theme_uri'  => $theme_data->parent()->display( 'ThemeURI', false, false ),
                'author'     => $theme_data->parent()->display( 'Author', false, false ),
                'author_uri' => $theme_data->parent()->display( 'AuthorURI', false, false ),
            );
        } else {
            $theme['template'] = '';
        }
        unset( $theme_template );


        $plugins       = array();
        $active_plugin = get_option( 'active_plugins' );
        foreach ( $active_plugin as $plugin_path ) {
            if ( ! function_exists( 'get_plugin_data' ) ) {
                require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            }

            $plugin_info = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin_path );

            $slug             = str_replace( '/' . basename( $plugin_path ), '', $plugin_path );
            $plugins[ $slug ] = array(
                'version'    => $plugin_info['Version'],
                'name'       => $plugin_info['Name'],
                'plugin_uri' => $plugin_info['PluginURI'],
                'author'     => $plugin_info['AuthorName'],
                'author_uri' => $plugin_info['AuthorURI'],
            );
        }
        unset( $active_plugins, $plugin_path );


        $data = array(
            'site'      => array(
                'domain'    => site_url(),
                'version'   => get_bloginfo( 'version' ),
                'multisite' => is_multisite(),
                'users'     => $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $wpdb->users INNER JOIN $wpdb->usermeta ON ({$wpdb->users}.ID = {$wpdb->usermeta}.user_id) WHERE 1 = 1 AND ( {$wpdb->usermeta}.meta_key = %s )", 'wp_' . $blog_id . '_capabilities' ) ),
                'lang'      => get_locale(),
            ),
            'pts'       => $pts,
            'comments'  => array(
                'total'    => $comments_count->total_comments,
                'approved' => $comments_count->approved,
                'spam'     => $comments_count->spam,
                'pings'    => $wpdb->get_var( "SELECT COUNT(comment_ID) FROM $wpdb->comments WHERE comment_type = 'pingback'" ),
            ),
            'options'   => $this->getServerInfo(),
            'theme'     => $theme,
            'plugins'   => $plugins,
        );

        $args = array(
            'body'      => $data,
            'blocking'  => false,
            'sslverify' => false,
        );

        if (!empty($filters))
        {
            $result = array();

            foreach($filters as $filter)
            {
                if (isset($data[$filter]))
                {
                    $result[] = $data[$filter];
                }
            }

            return $result;
        }

        return $data;
//        wp_remote_post( 'https://tracking.yoast.com/', $args );

    }


    function getServerInfo() {
        if ( function_exists( 'curl_version' ) ) {
            $curl = curl_version();
        } else {
            $curl = null;
        }


        $options = array(
            'site_db_charset'             => DB_CHARSET,

            'webserver_apache'            => $this->_is_apache() ? 1 : 0,
            'webserver_apache_version'    => function_exists( 'apache_get_version' ) ? apache_get_version() : 0,
            'webserver_nginx'             => $this->_is_nginx() ? 1 : 0,

            'webserver_server_software'   => $_SERVER['SERVER_SOFTWARE'],
            'webserver_gateway_interface' => $_SERVER['GATEWAY_INTERFACE'],
            'webserver_server_protocol'   => $_SERVER['SERVER_PROTOCOL'],

            'php_version'                 => phpversion(),

            'php_max_execution_time'      => ini_get( 'max_execution_time' ),
            'php_memory_limit'            => ini_get( 'memory_limit' ),
            'php_open_basedir'            => ini_get( 'open_basedir' ),

            'php_bcmath_enabled'          => extension_loaded( 'bcmath' ) ? 1 : 0,
            'php_ctype_enabled'           => extension_loaded( 'ctype' ) ? 1 : 0,
            'php_curl_enabled'            => extension_loaded( 'curl' ) ? 1 : 0,
            'php_curl_version_a'          => phpversion( 'curl' ),
            'php_curl'                    => ( ! is_null( $curl ) ) ? $curl['version'] : 0,
            'php_dom_enabled'             => extension_loaded( 'dom' ) ? 1 : 0,
            'php_dom_version'             => phpversion( 'dom' ),
            'php_filter_enabled'          => extension_loaded( 'filter' ) ? 1 : 0,
            'php_mbstring_enabled'        => extension_loaded( 'mbstring' ) ? 1 : 0,
            'php_mbstring_version'        => phpversion( 'mbstring' ),
            'php_pcre_enabled'            => extension_loaded( 'pcre' ) ? 1 : 0,
            'php_pcre_version'            => phpversion( 'pcre' ),
            'php_pcre_with_utf8_a'        => @preg_match( '/^.{1}$/u', 'ñ', $UTF8_ar ),
            'php_pcre_with_utf8_b'        => defined( 'PREG_BAD_UTF8_ERROR' ),
            'php_spl_enabled'             => extension_loaded( 'spl' ) ? 1 : 0,
        );

        return $options;
    }


    /**
     * Check if the web server is running on Apache
     * @return bool
     */
    function _is_apache() {
        if ( isset( $_SERVER['SERVER_SOFTWARE'] ) && stristr( $_SERVER['SERVER_SOFTWARE'], 'apache' ) !== false ) {
            return true;
        }
        return false;
    }

    /**
     * Check if the web service is running on Nginx
     *
     * @return bool
     */
    function _is_nginx() {
        if ( isset( $_SERVER['SERVER_SOFTWARE'] ) && stristr( $_SERVER['SERVER_SOFTWARE'], 'nginx' ) !== false ) {
            return true;
        }
        return false;
    }
}