var gulp        = require('gulp');                  // Gulp!
var fs          = require('fs');
var sass        = require('gulp-sass');             // Sass
var minifycss   = require('gulp-clean-css');       // Minify CSS
var uglify      = require('gulp-uglify');           // Uglify javascript
var rename      = require('gulp-rename');           // Rename files
var header      = require('gulp-header');           // Add header to files
var shell       = require('gulp-shell');
var babel       = require('gulp-babel');
var del         = require('del');
var moment = require('moment');
var color = require('ansi-colors');
var log = require('fancy-log');

var pkg     = require('./package.json');
var banner  = ['/**<%= pkg.name %>',
               '@version v<%= pkg.version %>',
               'Built: ' + moment().format('MMMM Do YYYY, h:mm:ss a') + '',
               '*/',
               ''].join(' ') + '\n';

//
//      Compile all CSS for the site
//
//////////////////////////////////////////////////////////////////////


gulp.task('styles', function (){
    var stream = gulp.src('resources/styles/scss/*.scss')                    // Build Our Stylesheet
        .pipe(sass({style: 'compressed', errLogToConsole: true}))  // Compile scss
        .pipe(rename({suffix: '.min'}))                              // Rename it
        .pipe(minifycss())                                         // Minify the CSS
        .pipe(header(banner, { pkg : pkg }))
        .pipe(gulp.dest('resources/styles'));                            // Set the destination to assets/css
    //        .pipe(livereload());                                       // Reloads server
    log(color.yellow('Sass compiled & minified'));  // Output to terminal

    return stream;
});

// Transpile JS with Bable

gulp.task('es6-amd', function()
{
    return gulp.src(['resources/scripts/src/*.js', 'resources/scripts/src/**/*.js'])
        .pipe(babel({"presets": ["@babel/preset-env"]}))
        .pipe(gulp.dest('resources/scripts/tmp'));
});


// Build RequireJS

gulp.task('build-js', shell.task(['npx r.js -o build-config.r.js']));

gulp.task('cleanup', (done) => {
    del(['resources/scripts/tmp']);
    done();
});

gulp.task('karma', shell.task(['karma start tests/karma/karma.conf.js']));

gulp.task('uglify-build-js', function(done)
{
    gulp.src('resources/scripts/build/hji.js', { allowEmpty: true })
        .pipe(rename({suffix: '.min'}))
        .pipe(uglify())
        .pipe(gulp.dest('resources/scripts/build'));
    done();
});

//
//      Codeception
//
//////////////////////////////////////////////////////////////////////
gulp.task('codeception', function() {
    gulp.src('codeception.yml')
        .pipe(codecept('', {notify: true, testSuite: 'wpunit'}))
        .on('error', notify.onError(testNotification('fail', 'wpunit')))
        .pipe(notify(testNotification('pass', 'wpunit')));
});
///////////////////////////////////////////////////////////////////////////////

//
//  Notification for codeception testing
//
//////////////////////////////////////////////////////////////////////
function testNotification(status, pluginName, override) {
    var options = {
        title: ( status == 'pass' ) ? 'Tests Passed' : 'Tests Failed',
        message: ( status == 'pass' ) ? '\n\nAll tests have passed!\n\n' : '\n\nOne or more tests failed...\n\n',
        icon: __dirname + '/node_modules/gulp-' + pluginName + '/assets/test-' + status + '.png'
    };
    options = _.merge(options, override);
    return options;
}


gulp.task('scripts', gulp.series('es6-amd', 'build-js', 'cleanup', 'uglify-build-js'));

function watch(){
    // Watch and run sass on changes
    gulp.watch("resources/styles/scss/**/*.scss", gulp.series('styles'));
    gulp.watch("resources/scripts/src/**/*.js", gulp.series('scripts'));

}

gulp.task('build', gulp.parallel('styles', 'scripts'));

try {
    fs.accessSync('./apigen.local');
    gulp.task('docs', shell.task(['apigen generate --debug --config ./apigen.local && php ./apigen/hji-hooks.php'], {verbose: true}));
} catch (err) {
    gulp.task('docs', () => {
        log(color.red('Can not run ApiGen Documentation Generator. Local config file does not exist. Follow these steps to setup: '));
        log(color.red('Copy apigen.neon to apigen.local. Edit the file and update destination parameter. Run "gulp docs" again.'));
    });

}

exports.watch = watch;
