

HJI.Membership.Admin = function ($) {
  var licenseKeyRegex = /^[A-Z0-9]{4}\-[A-Z0-9]{4}\-[A-Z0-9]{4}(\-[A-Z0-9]{4})?$/;
  var __obj = {
    onload: function onload() {
      if (hjiRequireJsConfig.is_wp_admin === true) {
        this.manageThemes(); // Force HJI CRM link to open in a new window, since it's an external link.

        $('#adminmenu .hji-crm-link').prop('target', '_blank');
      }
    },
    manageSettings: function manageSettings() {
      var lkey = jQuery('#hji-license-key');
      lkey.bind('change keyup', function (event) {
        var value = jQuery.trim(lkey.val());
        var valid = licenseKeyRegex.test(value);
        lkey[valid ? 'removeClass' : 'addClass']('hji-input-invalid');
        lkey.next('span')[valid ? 'hide' : 'show']();
      }).trigger('change');
      this.manageCustomerAssets();
    },

    /**
     * Auto-save for newly created widgets in the
     * Widgets admin dashboard
     *
     * To add a widget to autoSave call addAutoSaveWidget
     * trigger on window.load with the base_id of the widget
     *
     * $(window).load(function(){
     *  $(document).trigger('addAutoSaveWidget', 'widget_base_id');
     * });
     *
     * "widget_saved" hook will be triggered when the widget
     * is saved and widget object will be passed to the handler.
     */
    autoSaveWidgets: function autoSaveWidgets() {
      var autoSave = {
        widgets: [],
        getWidgets: function getWidgets() {
          return autoSave.widgets;
        },
        addWidget: function addWidget(widget) {
          autoSave.widgets.push(widget);
        },
        processWidget: function processWidget(infoObj) {
          // Base IDs of widgets to save
          var widgets = autoSave.getWidgets();
          var id = $(infoObj.widget).find('input[name="id_base"]').val();

          if ($.inArray(id, widgets) >= 0) {
            infoObj.save = true;
          }
        },
        setAjaxCompleteListener: function setAjaxCompleteListener() {
          jQuery(document).ajaxComplete(function (event, XMLHttpRequest, ajaxOptions) {
            if (typeof ajaxOptions.data == 'undefined') return false; // determine which ajax request is this (we're after "save-widget")

            var request = {},
                pairs = ajaxOptions.data.split('&'),
                i,
                split,
                widget;

            for (i in pairs) {
              split = pairs[i].split('=');
              request[decodeURIComponent(split[0])] = decodeURIComponent(split[1]);
            } // only proceed if this was a widget-save request


            if (request.action && request.action === 'save-widget') {
              // locate the widget block
              widget = jQuery('input.widget-id[value="' + request['widget-id'] + '"]').parents('.widget'); // trigger manual save, if this was the save request
              // and if we didn't get the form html response (the wp bug)

              if (!XMLHttpRequest.responseText) {
                // Trigger save only on widgets that indicated they want to be auto-saved
                var infoObj = {
                  widget: widget,
                  save: false
                }; // autoSaveWidget will update infoObj.save if necessary

                jQuery(document).trigger('autoSaveWidget', infoObj);
                wpWidgets.save(widget, 0, 1, 0); // we got an response, this could be either our request above,
                // or a correct widget-save call, so fire an event on which we can hook our js
              } else {
                jQuery(document).trigger('widget_saved', widget);
              }
            }
          });
        },
        setup: function setup() {
          autoSave.setAjaxCompleteListener();
          $(document).on('addAutoSaveWidget', function (e, widget) {
            autoSave.addWidget(widget);
          });
          $(document).on('autoSaveWidget', function (e, infoObj) {
            return autoSave.processWidget(infoObj);
          });
        }
      };
      autoSave.setup();
    },

    /**
     * Adds HJI themes to Appearance/Themes
     * as well as HJI Membership/Product Library/Themes
     *
     * @returns {boolean}
     */
    manageThemes: function manageThemes() {
      if (wp.themes) {
        var themes = wp.themes;

        if (!themes.data.hjiThemes) {
          $('.filter-links .hj-themes').trigger('click');
          var filterLinks = $('ul.filter-links');

          if (!filterLinks.length) {
            return false;
          }

          var li = $('<li><a href="#" data-sort="hj_themes">Home Junction Themes</a></li>').data('sort', 'hj_themes');
          filterLinks.prepend(li);
        } else {
          var RunCustomInstaller = themes.RunInstaller;

          RunCustomInstaller.init = function () {
            // Set up the view
            // Passes the default 'section' as an option
            this.view.browse('hj_themes');
          };

          RunCustomInstaller.init();
        }
      }
    },
    manageCustomerAssets: function manageCustomerAssets() {
      $('.upload-media-button').click(function (e) {
        e.preventDefault();
        var destinationSelector = $(this).data('destination');
        var imageSize = $(this).data('imageSize');
        if (!imageSize) imageSize = 'thumbnail';
        var image = wp.media({
          title: 'Upload Image',
          // multiple: true if you want to upload multiple files at once
          multiple: false
        }).open().on('select', function (e) {
          // This will return the selected image from the Media Uploader, the result is an object
          var uploaded_image = image.state().get('selection').first(); // We convert uploaded_image to a JSON object to make accessing it easier
          // Output to the console uploaded_image

          var sizeDetails = uploaded_image.toJSON().sizes[imageSize];
          if (!sizeDetails) sizeDetails = uploaded_image.toJSON().sizes['thumbnail'];
          if (!sizeDetails) sizeDetails = uploaded_image.toJSON().sizes['full'];
          var image_url = sizeDetails.url; // Let's assign the url value to the input field

          $(destinationSelector).val(image_url);
        });
      });
    }
  };
  $(window).on('load', function () {
    __obj.onload();
  });
  return __obj;
}(jQuery);