

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

hji.define(['jquery', 'underscore', 'hji-common/gMapsApi', 'hji-common/visibility', 'hji-common/gMapsEmbedApi'], function ($, _, GoogleMapsLoader, Visibility) {
  /*
   var geoData = [
       {
           lat: '', // Latitude
           lng: '', // Longitude
           info: '' // Information displayed inside infoWindow
       }
   ];
   */
  var gMap = function gMap(settings, mapOptions, callback) {
    var that = this;
    this.map = null; // Preset settings for ready() function

    this.config = $.extend({
      canvas: false,
      geoData: [],
      icon: false,
      drawingManager: false,
      autozoom: true,
      style: null
    }, settings);
    GoogleMapsLoader.load(HJI.commonVar('gMapsApiKey')).done(function () {
      var canvasVisibility = new Visibility(that.config.canvas);
      canvasVisibility.one('visible', function () {
        return that.init(settings, mapOptions, callback);
      });
    }).fail(function (arg) {
      if (arg.hasOwnProperty('gMapsApiKey') && !arg.gMapsApiKey) {
        if (!settings.geoData.length) {
          HJI.logError('Failed to load Google Map. Location is not provided.');
          return;
        } else if (settings.geoData.length > 1) {
          GoogleMapsLoader.load(HJI.commonVar('hjgMapsApiKey')).done(function () {
            return that.init(settings, mapOptions, callback);
          });
          return;
        }

        var gMapsEmbedApiKey = HJI.commonVar('hjgMapsApiKey');

        if (!gMapsEmbedApiKey) {
          HJI.logError('Failed to load Google Map. Google Maps Embed API Key is not provided.');
          return;
        }

        var address = function address() {
          if (HJI.rIDX && HJI.rIDX.getListing()) return HJI.rIDX.getListing().address;
          var propertyName = $(settings.geoData[0].info).text();

          if (propertyName.includes('–')) {
            var parts = propertyName.split('–');
            return parts[1].trim();
          }

          return propertyName || "".concat(settings.geoData[0].lat, ",").concat(settings.geoData[0].lng);
        };

        require(['hji-common/gMapsEmbedApi'], function (gMapsEmbedApi) {
          new gMapsEmbedApi({
            $el: settings.canvas,
            q: settings.geoData[0].address || address(),
            center: {
              lat: settings.geoData[0].lat,
              lng: settings.geoData[0].lng
            },
            zoom: 18,
            apiKey: gMapsEmbedApiKey,
            callback: callback
          }).renderMap();
        });
      }
    });
  };

  gMap.prototype.ready = function (callback) {
    if (typeof callback == 'function') {
      if (this.map) {
        callback();
      } else {
        $(this.config.canvas).on('gMapReady', function () {
          callback();
        });
      }
    }
  };

  gMap.prototype.init = function (settings, mapOptions, callback) {
    // Error flag
    this.errorFlag = false;
    this.config = $.extend({
      canvas: false,
      geoData: [],
      icon: false,
      drawingManager: false,
      autozoom: true
    }, settings || {});
    this.mapOptions = mapOptions || {}; // Currently defined shapes

    this.shapes = [];
    this.shapesIndex = 0; // autoincrement counter for shape IDs.

    this.callback = callback ? callback : undefined; // Terminate if canvas element is not defined

    if (!this.config.canvas || !jQuery(this.config.canvas).length) {
      this.gMapError('Canvas element for rendering Google Map was not defined.');
      return false;
    } else {
      $(this.config.canvas)[0].gMap = this;
      this.createMap();
    } // Utility similar to $.extend();


    function extend(a, b) {
      for (var key in b) {
        if (b.hasOwnProperty(key)) {
          a[key] = b[key];
        }
      }

      return a;
    }

    this.setPublicEvents();
    return this;
  };

  gMap.prototype.createMap = function () {
    var that = this;
    this.mapOptions = $.extend({
      center: new google.maps.LatLng(0, 0),
      zoom: 16,
      mapTypeId: google.maps.MapTypeId.ROADMAP,
      styles: null
    }, this.mapOptions);
    this.map = new google.maps.Map(this.config.canvas, this.mapOptions);
    this.markers = [];
    this.bounds = new google.maps.LatLngBounds();
    this.locationControlUI = false;

    if (this.config.geoData.length) {
      this.plotMarkers(this.config.geoData);
    }

    if (this.config.drawingManager) {
      var options = this.config.drawingManager.options ? this.config.drawingManager.options : null;
      var customControls = this.config.drawingManager.customControls ? this.config.drawingManager.customControls : null;
      var events = this.config.drawingManager.events ? this.config.drawingManager.events : null;
      this.drawingManager(options, customControls, events);
    }

    if (this.callback) {
      this.callback();
    }

    $(this.config.canvas).trigger('gMapReady');
    this.map.data.addListener('click', function (e) {
      if (e.feature) {
        if (confirm('Would you like to delete this polygon?')) {
          that.removeShape(e.feature);
        }
      }
    });
  };

  gMap.prototype.plotMarkers = function (locations) {
    var that = this;
    var infowindow = new google.maps.InfoWindow();
    var marker, i;

    for (i = 0; i < locations.length; i++) {
      var coords = new google.maps.LatLng(locations[i].lat, locations[i].lng);
      marker = new google.maps.Marker({
        position: coords,
        map: that.map,
        icon: this.getIcon(locations[i])
      });
      this.markers.push(marker);
      google.maps.event.addListener(marker, 'click', function (marker, i) {
        return function () {
          infowindow.setOptions({
            maxWidth: 350
          });
          infowindow.setContent('<div class="infoWindowContentWrap">' + locations[i].info);
          +'</div>';
          infowindow.open(that.map, marker);
        };
      }(marker, i));
      this.bounds.extend(coords);
    } // InfoWindow event - triggers when the window gets added to the DOM
    // Lazy-loads images and loads polyfill if necessary.


    google.maps.event.addListener(infowindow, 'domready', function () {
      require(['hji-common/unveil'], function () {
        $("body .infoWindowContentWrap img[data-src]").unveil(1, function () {
          // callback checks to see if we've included our object-fit polyfill
          if (typeof objectFitImages == 'function') {
            objectFitImages();
          }
        });
      });
    });
    this.map.fitBounds(this.bounds); // Keep zoom at 16 if bounds are zoomed-in too much
    // this usually happens when there is only 1 marker

    if (this.config.autozoom) {
      google.maps.event.addListenerOnce(that.map, "idle", function () {
        if (that.map.getZoom() > 16) that.map.setZoom(16);
      });
    }
  };

  gMap.prototype.refresh = function () {
    // Resize doesn't work very well when map is loaded inside tabs
    // - map is not zommed and positioned correctly
    google.maps.event.trigger(this.map, 'resize');
    this.map.fitBounds(this.bounds);
  };

  gMap.prototype.getIcon = function (location) {
    if (typeof location.icon != 'undefined' && location.icon) {
      return location.icon;
    } else if (this.config.icon) {
      // Is default icon set in the settings?
      return this.config.icon;
    }

    return null;
  };

  gMap.prototype.gMapError = function (error) {
    this.errorFlag = true;
    throw new Error(error);
  };

  gMap.prototype.publicEvents = function () {
    var that = this;
    var events = {
      refreshMap: function refreshMap() {
        setTimeout(function () {
          that.refresh();
        }, 25);
      }
    };
    return events;
  };

  gMap.prototype.setPublicEvents = function () {
    var events = this.publicEvents();

    if (events) {
      for (var e in events) {
        $(this.config.canvas).on(e, events[e]);
      }
    }
  };

  gMap.prototype.renderPolygons = function (polygons, options) {
    var that = this;
    polygons.forEach(function (polygon) {
      if (polygon.geometry) {
        that.renderGeoJson([polygon], Object.assign(that.config.style || {}, options || {}));
      } else {
        that.renderPolygon(polygon, Object.assign(that.config.style || {}, options || {}));
      }
    });
  };
  /**
   * Renders polygon on the map
   * @param coords - array of lat/lng pairs
   * Example:
   * [
   *      [32.837481,-117.280169],
   *      [32.837481,-117.280169],
   *      [32.838491,-117.267809]
   * ]
   */


  gMap.prototype.renderPolygon = function (coords, options) {
    var that = this;
    this.ready(function () {
      // Define polygon
      var polygon;
      var polygonCoords = []; // Settings bounds on cities (like Scottsdale)
      // fails if there are no shapes on the map
      // Getting bounds on every shape render
      // resets zoom only to that particular shape.

      if (that.shapes.length == 0) {
        that.bounds = new google.maps.LatLngBounds();
      }

      for (var i in coords) {
        // Supporting both lat/lng object as well as simple array
        if (coords[i].lat && coords[i].lng) {
          var latLng = new google.maps.LatLng(coords[i].lat, coords[i].lng);
        } else {
          var latLng = new google.maps.LatLng(coords[i][0], coords[i][1]);
        }

        polygonCoords.push(latLng);
        that.bounds.extend(latLng);
      } // Construct polygon


      var settings = $.extend({
        paths: polygonCoords,
        editable: true,
        collectable: true,
        hjiPolygonObj: null
      }, options);
      polygon = new google.maps.Polygon(settings);

      if (settings.collectable) {
        if (settings.hjiPolygonObj) {
          polygon.hjiPolygonObj = settings.hjiPolygonObj;
        }

        polygon.shapeId = that.collectShape(polygon);
        polygon.type = 'polygon';
        that.setShapeListeners(polygon);
      } // Render polygon


      polygon.setMap(that.map);
      that.shapesUpdatedTrigger();
    });
  };

  gMap.prototype.renderGeoJson = function (polygons, options) {
    var that = this;
    this.ready(function () {
      // Construct polygon
      polygons.forEach(function (polygon) {
        var features = that.map.data.addGeoJson({
          type: "Feature",
          properties: {
            areaType: polygon.areaType || null,
            geoType: polygon.geoType || null,
            id: polygon.id || null,
            label: polygon.label || null,
            name: polygon.name || null
          },
          geometry: polygon.geometry
        }, {
          collectable: true,
          hjiPolygonObj: null
        });
        if (that.config.style) that.map.data.setStyle(that.config.style);
      });
      that.shapesUpdatedTrigger();
    });
  };

  gMap.prototype.drawingManager = function (options, customControls, customEvents) {
    var that = this,
        settings = $.extend({
      // drawingMode: google.maps.drawing.OverlayType.MARKER,
      drawingControl: true,
      drawingControlOptions: {
        position: google.maps.ControlPosition.TOP_CENTER,
        drawingModes: [//                            google.maps.drawing.OverlayType.CIRCLE,
        google.maps.drawing.OverlayType.POLYGON, google.maps.drawing.OverlayType.RECTANGLE]
      },
      circleOptions: Object.assign({
        clickable: true,
        editable: true,
        zIndex: 1
      }, that.config.style || {}),
      rectangleOptions: Object.assign({
        editable: true
      }, that.config.style || {}),
      // rectangles created are editable by default
      polygonOptions: Object.assign({
        editable: true
      }, that.config.style || {})
    }, options || {});
    var controls = [this.locationButton()]; // Merge customControls with existing controls

    if (customControls) {
      controls = controls.concat(customControls);
    }

    this.ready(function () {
      var drawingManager = new google.maps.drawing.DrawingManager(settings);
      google.maps.event.addListener(drawingManager, 'overlaycomplete', function (e) {
        drawingManager.setDrawingMode(null); // that.removeCircle();

        var newShape = e.overlay;

        if (e.type === google.maps.drawing.OverlayType.RECTANGLE) {
          newShape.type = 'rectangle';
        } else if (e.type === google.maps.drawing.OverlayType.CIRCLE) {
          newShape.type = 'circle';
        } else if (e.type === google.maps.drawing.OverlayType.POLYGON) {
          newShape.type = 'polygon';
        }

        newShape.shapeId = that.collectShape(newShape);
        that.setShapeListeners(newShape);
        that.shapesUpdatedTrigger();
      }); // Create custom events

      var eventSetter = function eventSetter(eventName, eventHandler) {
        google.maps.event.addListener(drawingManager, eventName, function (e) {
          eventHandler();
        });
      };

      if (customEvents) {
        // Loop through events
        for (var i in customEvents) {
          var event = customEvents[i].event,
              handler = customEvents[i].handler;
          eventSetter(event, handler);
        }
      }

      drawingManager.setMap(that.map); // If array - loop through and add controls to the drawing manager

      if (_typeof(controls) == 'object') {
        for (var i in controls) {
          controls[i].index = 1;
          that.map.controls[google.maps.ControlPosition.TOP_CENTER].push(controls[i]);
        }
      }
    });
  };

  gMap.prototype.shapesUpdatedTrigger = function () {
    var that = this;
    $(this.config.canvas).trigger('shapesUpdated', {
      shapes: that.shapes,
      getUrlValues: function getUrlValues() {
        var values = {};
        var result = {};

        for (var i in that.shapes) {
          var type = that.shapes[i].type == 'rectangle' || that.shapes[i].type == 'polygon' ? 'polygon' : that.shapes[i].type;
          if (typeof type === 'undefined') continue;

          if (!values[type]) {
            values[type] = [];
          }

          values[type].push(that.toUrlValue(that.shapes[i]));
        }

        if (!_.isEmpty(values)) {
          for (var i in values) {
            if (values[i].length) {
              result[i] = JSON.stringify(values[i]);
            }
          }
        }

        return result;
      },
      gMap: that
    });
  };

  gMap.prototype.collectShape = function (shape) {
    this.shapesIndex++;
    shape.shapeId = this.shapesIndex;
    this.shapes.push(shape);
    return shape.shapeId;
  };

  gMap.prototype.unCollectShape = function (shape, callback) {
    for (var i in this.shapes) {
      if (this.shapes[i].shapeId == shape.shapeId) {
        $(this.config.canvas).trigger('shapeDelete', shape);
        this.shapes.splice(i, 1);
        this.shapesUpdatedTrigger();
      }
    }

    if (typeof callback != 'undefined') {
      callback();
    }
  };

  gMap.prototype.setShapeListeners = function (shape) {
    var that = this;

    if (shape.type == 'polygon') {
      google.maps.event.addListener(shape.getPath(), 'set_at', function () {
        // that.shapes[shape.shapeId] = shape;
        that.updateShape(shape);
        that.shapesUpdatedTrigger();
        $(that.config.canvas).trigger('shapeBoundsChanged', shape);
      });
      google.maps.event.addListener(shape.getPath(), 'insert_at', function () {
        // that.shapes[shape.shapeId] = shape;
        that.updateShape(shape);
        that.shapesUpdatedTrigger();
        $(that.config.canvas).trigger('shapeBoundsChanged', shape);
      });
    } else if (shape.type == 'rectangle') {
      google.maps.event.addListener(shape, 'bounds_changed', function () {
        // that.shapes[shape.shapeId] = shape;
        that.updateShape(shape);
        that.shapesUpdatedTrigger();
        $(that.config.canvas).trigger('shapeBoundsChanged', shape);
      });
    } else if (shape.type == 'circle') {
      google.maps.event.addListener(shape, 'radius_changed', function () {
        // that.shapes[shape.shapeId] = shape;
        that.updateShape(shape);
        that.shapesUpdatedTrigger();
        $(that.config.canvas).trigger('shapeBoundsChanged', shape);
      });
      google.maps.event.addListener(shape, 'center_changed', function () {
        // that.shapes[shape.shapeId] = shape;
        that.updateShape(shape);
        that.shapesUpdatedTrigger();
        $(that.config.canvas).trigger('shapeBoundsChanged', shape);
      });
    }

    google.maps.event.addListener(shape, 'click', function (e) {
      if (confirm('Would you like to delete this polygon?')) {
        that.removeShape(shape);
      }
    });
  };

  gMap.prototype.updateShape = function (shape) {
    var that = this;

    _.each(this.shapes, function (v, k, list) {
      if (v.shapeId == shape.shapeId) {
        that.shapes[k].shape = shape;
      }
    });
  };
  /**
   * Removes shape from the map
   *
   * @param shape - object - Google Maps shape object
   */


  gMap.prototype.removeShape = function (shape) {
    var that = this;

    if (typeof shape.setMap === 'function') {
      shape.setMap(null);
      this.unCollectShape(shape, function () {
        if (shape.type == 'circle') {
          $(that.config.canvas).trigger('userLocationRemoved');
        }

        if (_.isObject(shape.hjiPolygonObj) && _.has(shape.hjiPolygonObj, 'id')) {
          $(that.config.canvas).trigger('polygon_removed', {
            id: shape.hjiPolygonObj.id,
            shape: shape
          });
        }
      });
    } else if (typeof shape.getGeometry === 'function') {
      var data = {
        id: shape.getProperty('id'),
        geoJSON: null,
        shape: shape
      },
          _triggerEvent = function _triggerEvent(data) {
        $(that.config.canvas).trigger('polygon_removed', data);
      };

      if (data.id) {
        shape.toGeoJson(function (obj) {
          data.geoJSON = obj;

          _triggerEvent(data);

          that.map.data.remove(shape);
        });
      } else {
        this.map.data.remove(shape);
        $(that.config.canvas).trigger('polygon_removed', data);
      }
    }
  };

  gMap.prototype.locationButton = function () {
    var that = this,
        controlDiv = document.createElement('div'); // Set CSS styles for the DIV containing the control
    // Setting padding to 5 px will offset the control
    // from the edge of the map

    controlDiv.style.marginTop = '5px';
    controlDiv.style.marginLeft = '-5px';
    controlDiv.style.padding = '4px';
    controlDiv.style.boxShadow = '0px 1px 4px -1px rgba(0, 0, 0, 0.298039)';
    controlDiv.style.backgroundColor = 'white';
    controlDiv.style.zIndex = '11'; // Set CSS for the control border

    var controlUI = document.createElement('div');
    controlUI.style.backgroundColor = 'white';
    controlUI.style.border = 'none';
    controlUI.style.borderTopRightRadius = '2px';
    controlUI.style.borderBottomRightRadius = '2px';
    controlUI.style.cursor = 'pointer';
    controlUI.style.textAlign = 'center';
    controlUI.title = 'Search near your current location';
    controlDiv.appendChild(controlUI); // Set CSS for the control interior

    var controlText = document.createElement('div'); //        controlText.style.fontFamily = 'Arial,sans-serif';

    controlText.style.fontSize = '14px';
    controlText.style.color = 'rgb(142, 142, 142)';
    controlText.style.width = '16px';
    controlText.style.height = '16px';
    controlText.style.lineHeight = '16px'; //        controlText.innerHTML = 'Current Location';

    controlText.className = 'fa fa-crosshairs';
    this.locationControlUI = controlUI;
    controlUI.appendChild(controlText);
    google.maps.event.addDomListener(controlUI, 'click', function () {
      $(that.config.canvas).trigger('locationControlClick', controlUI);
      controlText.style.color = 'rgb(0,0,0)';
      that.showUserLocation();
    });

    if (navigator.geolocation) {
      return controlDiv;
    }

    return false;
  };

  gMap.prototype.showUserLocation = function (callback, error) {
    var that = this;

    var userPositionCallback = function userPositionCallback(position) {
      var pos = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);

      if (typeof callback != 'undefined') {
        callback(position);
      }

      var circle = {
        map: that.map,
        center: pos,
        radius: that.toMeters(0.5),
        'clickable': true,
        'zIndex': 1
      };
      that.map.setCenter(pos); // Remove existing polygons

      that.removeAllShapes(); // Render circle

      var locationCircle = new google.maps.Circle(circle); // Trigger event that nearbyCircleRender

      $(that.config.canvas).trigger('nearbyCircleRender');
      that.map.fitBounds(locationCircle.getBounds());
      var newShape = locationCircle;
      newShape.type = 'circle';
      newShape.shapeId = that.collectShape(newShape);
      that.setShapeListeners(newShape); // Trigger user location event with location information

      $(that.config.canvas).trigger({
        type: 'userLocationFound',
        coords: position.coords
      });
      that.shapesUpdatedTrigger();
    };

    require(['hji-common/getUserLocation'], function (getUserLocation) {
      new getUserLocation(userPositionCallback, error);
    });

    return false;
  };

  gMap.prototype.renderCircle = function (options) {
    var pos = new google.maps.LatLng(options.coords.lat, options.coords.lng);
    var circle = Object.assign({
      map: this.map,
      center: pos,
      radius: this.toMeters(options.radius),
      editable: true
    }, this.config.style || {});
    this.map.setCenter(pos);
    var locationCircle = new google.maps.Circle(circle);
    var newShape = locationCircle;
    newShape.type = 'circle';
    newShape.shapeId = this.collectShape(newShape);
    this.setShapeListeners(newShape);
    this.shapesUpdatedTrigger();
    return locationCircle;
  };

  gMap.prototype.removeAllShapes = function () {
    // user location passes a circle
    // we don't support circles and polygons together,
    // so circles will need to be removed
    // remove features/geojson
    this.map.data.setMap(null); // hides polygons, but doesn't remove from data collection
    // remove collected shapes

    var l = this.shapes.length;

    for (var i = 0; i < l; i++) {
      if (typeof this.shapes[i].setMap === 'function') {
        this.shapes[i].setMap(null); // delete this.shapes[i];

        delete this.shapes[i];
      } else if (typeof this.shapes[i].getGeometry === 'function') {
        this.map.data.remove(this.shapes[i]);
        delete this.shapes[i];
      }
    } // because we only deleted items, the length of the array is still preserved
    // therefore we're resetting the array


    this.shapes = [];
  };

  gMap.prototype.removeCircle = function () {
    var that = this; // user location passes a circle
    // we don't support circles and polygons together,
    // so circles will need to be removed

    for (var i in this.shapes) {
      if (this.shapes[i].type == 'circle') {
        this.shapes[i].setMap(null); // delete this.shapes[i];

        this.shapes.splice(i, 1);
        $(that.config.canvas).trigger('userLocationRemoved');
        this.shapesUpdatedTrigger();
      }
    }
  };

  gMap.prototype.fitBounds = function () {
    this.map.fitBounds(this.bounds);
  };

  gMap.prototype.toMeters = function (miles) {
    var meters = miles * 1000 / 0.621271;
    return Math.round(meters);
  };
  /**
   * String separators:
   * pipe, space, comma, colon
   *
   * @param string
   */


  gMap.prototype.strToLatLng = function (string, separator) {
    if (separator) {
      var a = string.split(separator);

      if (parseFloat(a[0]) > 0) {
        return this.latLng(a[0], a[1]);
      } else {
        return this.latLng(a[1], a[0]);
      }
    }

    if (string.indexOf('|') != -1) return this.strToLatLng(string, '|');
    if (string.indexOf(' ') != -1) return this.strToLatLng(string, ' ');
    if (string.indexOf(',') != -1) return this.strToLatLng(string, ',');
    if (string.indexOf(':') != -1) return this.strToLatLng(string, ':');
  };

  gMap.prototype.latLng = function (lat, lng) {
    return {
      lat: lat,
      lng: lng
    };
  };

  gMap.prototype.toUrlValue = function (shape) {
    var coords,
        arr = [];
    coords = this.getCoords(shape);

    for (var i in coords) {
      arr.push(coords[i].lat + ',' + coords[i].lng);
    }

    if (shape.type == 'circle') {
      var circle = this.getCircle(shape);
      return circle.center.lat + ':' + circle.center.lng + ':' + (circle.radius * 0.000621371).toFixed(2);
    } else {
      return arr.join(',');
    }
  };

  gMap.prototype.getCoords = function (shape) {
    var item;

    switch (shape.type) {
      case 'polygon':
        item = this.getPolygon(shape);
        return item.coords;

      case 'rectangle':
        item = this.getRectangle(shape);
        return item.coords;

      case 'cirlce':
        item = this.getCircle(shape);
        return item.coords;

      case 'box':
        item = this.getBox(shape);
        return item.coords;
    }
  };

  gMap.prototype.getRectangle = function (shape) {
    var rectangle = {
      coords: []
    };
    rectangle.coords.push(this.latLng(shape.getBounds().getNorthEast().lat(), shape.getBounds().getNorthEast().lng()));
    rectangle.coords.push(this.latLng(shape.getBounds().getSouthWest().lat(), shape.getBounds().getNorthEast().lng()));
    rectangle.coords.push(this.latLng(shape.getBounds().getSouthWest().lat(), shape.getBounds().getSouthWest().lng()));
    rectangle.coords.push(this.latLng(shape.getBounds().getNorthEast().lat(), shape.getBounds().getSouthWest().lng())); // Close polygon

    rectangle.coords = this.closePolygon(rectangle.coords);
    return rectangle;
  };

  gMap.prototype.getPolygon = function (shape) {
    var that = this;
    var polygon = {
      coords: []
    };
    var path = shape.getPath();

    if (shape.hjiPolygonObj) {
      polygon.id = shape.hjiPolygonObj.id;
      polygon.hjiData = shape.hjiPolygonObj;
    }

    path.forEach(function (e, i) {
      polygon.coords.push(that.latLng(e.lat(), e.lng()));
    }); // Close polygon

    polygon.coords = this.closePolygon(polygon.coords);
    return polygon;
  };

  gMap.prototype.getBox = function (shape) {};

  gMap.prototype.getCircle = function (shape) {
    var circle = {
      center: this.latLng(shape.getCenter().lat(), shape.getCenter().lng()),
      radius: shape.getRadius()
    };
    return circle;
  };

  gMap.prototype.closePolygon = function (coords) {
    if (coords.length > 1) {
      var lastPair = coords[coords.length - 1];

      if (JSON.stringify(coords[0]) !== JSON.stringify(lastPair)) {
        coords.push(coords[0]);
      }
    }

    return coords;
  };

  return gMap;
});