

hji.define(['jquery', 'backbone'], function ($, Backbone) {
  var InlineModalView = Backbone.View.extend({
    initialize: function initialize() {
      var that = this;
      this.bind("ok", this.okClicked);
      this.bind("onCancel", this.onCancel);
      this.bind("onSubmit", this.onSubmit);
      this.bind("onError", this.onError);
      this.bind("onSuccess", this.onSuccess);
      this.template = this.model.get('content'); // Custom on hidden event - to cover:
      // - Cancel btn click
      // - Close X click
      // - Submit btn click (we remove this on submit though)
      // - Click outside of the modal boundaries

      $(document).off('hide.bs.modal.gfmodal', '.modal');
      $(document).on('hide.bs.modal.gfmodal', '.modal', {
        view: this
      }, function (e) {
        //that.reattach();
        that.trigger('onCancel');
      });
      this.delegateEvents();
    },
    events: {
      "onCancel": "onCancel",
      "onSubmit": "onSubmit",
      "onError": "onError",
      "onSuccess": "onSuccess"
    },
    onCancel: function onCancel() {
      this.reattach();
    },
    onSubmit: function onSubmit() {
      // Don't want to be executing this when OK button is clicked.
      $(document).off('hide.bs.modal.gfmodal', '.modal'); // Modal is closing, so we need to re-attach the content
      // back to the original place in the DOM

      this.reattach();
    },
    onError: function onError(data) {},
    onSuccess: function onSuccess(data) {},
    okClicked: function okClicked(modal) {
      this.trigger('onSubmit');
    },
    render: function render() {
      this.$el.html(this.template);
      return this;
    },

    /**
     * Reattaches modal's content to its original spot in the DOM
     *
     * @returns {ModalView}
     */
    reattach: function reattach() {
      // Re-attache content of the form
      $(this.model.get('selector')).append(this.template);
      return this;
    },

    /**
     * Reattaches original gravity form HTML to its original sport in the DOM
     *
     * Note: because ajax-enabled gravity form modifies its content,
     * we need to be able to re-set the form to the initial state after user done
     * working with it.
     */
    reattachOriginal: function reattachOriginal() {
      var that = this; // Clear out existing content in the DOM

      $(this.model.get('selector')).html(''); // Apply modifications to original content

      modifyOriginalContent(that.model.get('original'));
      setTimeout(function () {
        // Append original content into the DOM
        $(that.model.get('selector')).append(that.model.get('original')); // Refresh gravity form model, since content has changed

        that.model.refresh();
      }, 25);
    }
  });

  function modifyOriginalContent($content) {
    // Fix datepicker within original content that is being re-attached
    // it's a known issue that datepicker doesn't clone well.
    $content.find('.datepicker').each(function () {
      refreshDatepicker($(this));
    });
  }
  /**
   * Refreshes datepicker element with its current settings
   *
   * @param $datepicker jQuery object
   */


  function refreshDatepicker($datepicker) {
    var dpSettings = $datepicker.data('datepicker').settings,
        $img = $datepicker.next('img.ui-datepicker-trigger'); // Destroy datepicker

    $datepicker.datepicker('destroy'); // Remove datepicker icon

    if ($img.length) {
      $img.remove();
    } // Initialize datepicker


    $datepicker.datepicker(dpSettings);
  }

  ;
  return InlineModalView;
});