

hji.define(['require', 'jquery', 'underscore', 'backbone', './contentModelFactory', './gfModal', './inlineModalView'], function (require, $, _, Backbone, ContentModelFactory) {
  var modals = {};
  /**
   * Initializes Modals for all [data-init="modal"] attributes on the page
   */

  modals.initModals = function () {
    this.gforms = require('./gfModal');
    var triggers = $('[data-init="modal"]');
    var targets = [];
    this.options = {
      className: 'hjitb modal hji-modal backbone-modal',
      title: null,
      okText: 'OK',
      focusOk: true,
      okCloses: true,
      cancelText: 'Cancel',
      showFooter: true,
      allowCancel: true,
      escape: true,
      animate: false,
      enterTriggersOk: false
    };

    _.each(triggers, function (el, index, list) {
      targets.push($(el).data('target'));
    });

    targets = _.unique(targets);
    triggers.each(function () {
      modals.setTriggerEvents(this);
    });
    modals.setOnLoadEvents(targets);
  };
  /**
   * Sets modal trigger events for identified content via data-init attributes.
   *
   * @param el string - element selector
   */


  modals.setTriggerEvents = function (el) {
    // On click
    $(el).on('click', function (e) {
      e.preventDefault();
      var target = $(this).data('target');
      var content = $(target);

      if (content.length) {
        modals.createModal(target);
      }
    });
  };
  /**
   * Create modals on document ready
   * if content has to be displayed as soon as page loads.
   *
   * Note: only 1 on load event per TARGET - there are can be
   * multiple triggers for each target though.
   */


  modals.setOnLoadEvents = function (targets) {
    // GravityForms onload events
    this.gforms.setOnLoadEvents();
  };
  /**
   * Creates and opens Modal for an inline content
   *
   * @param content
   * @param target
   */


  modals.createModal = function (target) {
    if (target instanceof Backbone.Model) {
      var contentModel = target;
    } else {
      var contentModel = modals.getContentModel(target);
    }

    var ModalView = modals.createView(contentModel); // Is gravity form?

    if (contentModel.has('form')) {
      var defaultOptions = {
        //className:  'hjitb modal hji-modal backbone-modal',
        content: new ModalView({
          model: contentModel
        }),
        okText: contentModel.get('form').submitTxt,
        animate: true,
        allowCancel: true
      };
      var modal = new Backbone.BootstrapModal(_.extend(defaultOptions, this.getOptions(contentModel.get('$el')))).open();
    } else {
      // TODO: refactor so className gets combined and options don't get wrongfully overwritten
      var defaultOptions = {
        //className: 'hjitb modal hji-modal backbone-modal',
        content: new ModalView({
          model: contentModel
        }),
        animate: true
      };
      var modal = new Backbone.BootstrapModal(_.extend(defaultOptions, this.getOptions(contentModel.get('$el')))).open();
    }
  };
  /**
   * Options parsed from data attributes and merged with default this.options.
   *
   * @param $el
   * @returns {{className: string, title: null, okText: string, focusOk: boolean, okCloses: boolean, cancelText: string, showFooter: boolean, allowCancel: boolean, escape: boolean, animate: boolean, enterTriggersOk: boolean}|*}
   */


  modals.getOptions = function ($el) {
    var _dataOptions = $el.data();

    var _options = this.options;
    if (!_.isObject(_dataOptions) || _.isEmpty(_dataOptions)) return _options;

    for (var key in _dataOptions) {
      var val = _dataOptions[key];

      if (key == 'className') {
        _options.className = _options.className + ' ' + _dataOptions.className;
      } else {
        _options[key] = val;
      }
    }

    return _options;
  };
  /**
   * Returns Gravity Form content itemized into an object
   *
   * @param target string - content selector from where the content is being retrieved
   * @returns {*}
   */


  modals.getContentModel = function (target) {
    var contentModelFactory = new ContentModelFactory({
      selector: target
    });
    var contentModel = contentModelFactory.create();
    return contentModel;
  };
  /**
   * Returns content View object for the BootstrapModal
   *
   * @param contentModel
   * @returns {*}
   */


  modals.createView = function (contentModel) {
    if (contentModel.has('form')) {
      return this.gforms.createGFView();
    } else {
      return require('./inlineModalView');
    }
  };

  return modals;
});