

// Docu : http://wiki.moxiecode.com/index.php/TinyMCE:Create_plugin/3.x#Creating_your_own_plugins
function createTinyMCEWidget(pluginModel) {
  tinymce.create('tinymce.plugins.' + pluginModel.handle, {
    /**
     * Initializes the plugin, this will be executed after the plugin has been created.
     * This call is done before the editor instance has finished it's initialization so use the onInit event
     * of the editor instance to intercept that event.
     *
     * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
     * @param {string} url Absolute URL to where the plugin is located.
     */
    init: function init(ed, url) {
      // Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand('mce_FeaturedListings');
      ed.addCommand('mce_' + pluginModel.handle, function () {
        ed.windowManager.open({
          file: ajaxurl + '?action=' + pluginModel.action + '&plugin=tinymce-' + pluginModel.handle,
          width: pluginModel.width + ed.getLang(pluginModel.handle + '.delta_width', 0),
          height: pluginModel.height + ed.getLang(pluginModel.handle + '.delta_height', 0),
          inline: 1
        }, {
          plugin_url: url // Plugin absolute URL

        });
      });
      ed.addButton(pluginModel.handle, {
        title: pluginModel.name,
        cmd: 'mce_' + pluginModel.handle,
        image: pluginModel.iconUrl
      });
    },

    /**
     * Creates control instances based in the incoming name. This method is normally not
     * needed since the addButton method of the tinymce.Editor class is a more easy way of adding buttons
     * but you sometimes need to create more complex controls like listboxes, split buttons etc then this
     * method can be used to create those.
     *
     * @param {String} n Name of the control to create.
     * @param {tinymce.ControlManager} cm Control manager to use in order to create new control.
     * @return {tinymce.ui.Control} New control instance or null if no control was created.
     */
    createControl: function createControl(n, cm) {
      return null;
    },

    /**
     * Returns information about the plugin as a name/value array.
     * The current keys are longname, author, authorurl, infourl and version.
     *
     * @return {Object} Name/value array containing information about the plugin.
     */
    getInfo: function getInfo() {
      return {
        longname: pluginModel.name,
        author: 'Home Junction Inc.',
        authorurl: 'http://spatialmatch.com',
        infourl: 'http://spatialmatch.com',
        version: "1.0"
      };
    }
  }); // Register plugin

  tinymce.PluginManager.add(pluginModel.handle, tinymce.plugins[pluginModel.handle]);
}

jQuery(document).ready(function () {
  if (hjiSettings && hjiSettings.tinymcePlugins) {
    for (var i in hjiSettings.tinymcePlugins) {
      createTinyMCEWidget(hjiSettings.tinymcePlugins[i]);
    }
  }
});