

hji.define(['require', 'jquery', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, Widget) {
  /**
   * Area Details Widget
   *
   * Parameters:
   *      @param data-init string         widgets/areaDetails
   *      @param data-id string           Area ID
   *      @param data-chooser boolean     Render Chooser true|false
   *      @param data-chooser-limit       number of items to be returned within the chooser
   *
   * Example:
   *      <div data-init="widgets/areaDetails"></div>
   *
   *      Widget Object can be accessed in the DOM as follows:
   *          $('[data-init="widgets/areaDetails"]')[0].widget;
   *
   * Manipulating widget instance:
   *      // Setting new area ID
   *      $('[data-init="widgets/areaDetails"]')[0].widget.options.id = 'ac28dd4fdde34d4040931bca6459ceea';
   *      // Re-rendering the whole widget
   *      $('[data-init="widgets/areaDetails"]')[0].render();
   */
  var areaDetails = function areaDetails() {
    this.$el = null;
    this.params = {};
    this.chooser = null;
    this.area = null;
    this.details = null;
    this.options = {
      id: null,
      chooser: true,
      chooserLimit: 15
    };
  };

  areaDetails.prototype = new Widget();

  areaDetails.prototype.render = function () {
    // Reset object instance, so it can be easily re-rendered with set options.
    this.reset();
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/areas'], function () {
        if (that.options.chooser) that.loadChooser();
        if (that.options.id) that.loadArea(that.options.id);
      });
    });
  };

  areaDetails.prototype.loadChooser = function () {
    var that = this;

    require(['hji-common/widgets/chooser'], function (Chooser) {
      var options = {
        limit: that.options.chooserLimit
      };
      Chooser.init(that.$el.find('.chooser-wrapper'), options);
      Chooser.onSelect(function (id) {
        that.loadArea(id);
      });
    });
  };

  areaDetails.prototype.loadArea = function (id) {
    var that = this;

    window.require(['slipstream/areas/widget/details', 'slipstream/areas/model/area'], function (Details, Area) {
      /*
       * An /areas/model/area keeps track of the selected area.
       *
       * https://slipstream.homejunction.com/v1/docs/js/areas/model/area
       */
      if (!that.area) that.area = new Area();
      /*
       * Make sure we are not already displaying this area.    There is no
       * reason to refresh the page unless we absolutely have to.
       */

      var nowShowing = that.area.get('id');

      if (!id || nowShowing === id) {
        return; // Nothing to do!
      }
      /*
       * We now fetch the selected area.
       *
       * https://slipstream.homejunction.com/v1/docs/js/areas/model/area
       */


      that.area.fetch({
        parameters: {
          id: id,
          details: true,
          geometry: true
        }
      });
      this.details = Details.init({
        renderTo: that.$el.find('.details-wrapper'),
        model: that.area
      });
    });
  };

  areaDetails.prototype.template = function () {
    var template = $('<div class="slipstream">').append($('<div class="slipstream-areas">').append($('<div class="details-wrapper">')));
    if (this.options.chooser) template.find('.slipstream-areas').prepend('<div class="chooser-wrapper">');
    return template;
  };
  /**
   * Resets areaWidget instance so it can be re-rendered.
   */


  areaDetails.prototype.reset = function () {
    this.area = null;
  };

  return areaDetails;
});