

hji.define(['jquery'], function ($) {
  /**
   * Area Chooser Widget
   *
   * Auto-complete area selector.
   *
   * Usage:
   *      Chooser.int($target, options);
   *      Chooser.onSelect(function(areaId){ callback(areaId); );
   */
  return {
    events: {
      onSelect: 'hji.widgets.chooser.onSelect'
    },
    chooser: {},
    settings: {},
    $el: null,
    init: function init($renderTo, options) {
      this.$el = $renderTo;
      this.settings.bias = HJI.commonVar('areaBias');
      this.settings = $.extend(this.settings, options);
      this.run();
      return this;
    },
    run: function run() {
      var that = this;

      window.require(['slipstream/areas/widget/chooser'], function (Chooser) {
        /*
         * Create the area chooser widget where the user will select the
         * area whose details they want to view.
         *
         * https://slipstream.homejunction.com/v1/docs/js/areas/widget/chooser
         */
        var chooserOptions = {
          // renderTo: that.$el.find('.chooser-wrapper'),
          renderTo: that.$el,
          parameters: that.settings
        };
        that.chooser = Chooser.init(chooserOptions);
        /*
         * The model associated with the /areas/widget/chooser will always contain the
         * area that the user selected. Let's watch that model for changes.
         *
         * https://slipstream.homejunction.com/v1/docs/js/areas/model/area
         */

        that.chooser.model.on('change', function () {
          /*
           * At this point the user has selected an area.  Get the ID of the selected area.
           */
          var id = that.chooser.model.get('id');

          if (id) {
            that.$el.trigger(that.events.onSelect, id);
          }
        });
      });
    },
    onSelect: function onSelect(callback) {
      if (typeof callback !== 'function') {
        console.error('Callback is not a function.');
        return false;
      }

      this.$el.on(this.events.onSelect, function (event, areaId) {
        callback(areaId);
      });
      return this;
    }
  };
});