

hji.define('hji-common/widgets/widget',['jquery'], function ($) {
  /**
   * Abstract Slipstream Widget Class
   *
   * @constructor
   */
  var Widget = function Widget() {
    this.moduleName = 'widget';
    this.$el = null;
    this.options = {};
  };

  Widget.prototype.init = function ($el, options) {
    this.$el = $el;
    this.$el[0][this.moduleName] = this;
    $.extend(this.options, options);
    this.render();
  };
  /**
   * Abstract Render Method
   */


  Widget.prototype.render = function () {
    console.error('widget.render() is not implemented.');
  };

  Widget.prototype.template = function () {
    var template = $('<div class="slipstream">').append($('<div class="slipstream-areas">'));
    if (this.options.chooser) template.find('.slipstream-areas').prepend('<div class="chooser-wrapper">');
    return template;
  };

  return Widget;
});


var baseUrl = HJI.get('common').apiBaseUrl ? HJI.get('common').apiBaseUrl : 'https://slipstream-cdn.homejunction.com';
var nationalApiUrl = HJI.commonVar('env') == 'production' ? 'https://national-reports-cdn.homejunction.com' : 'https://national-reports-test-cdn.homejunction.com';
hji.define('hji-common/slipstream',['slipstream/bootstrap'], function () {
  if (!HJI || !HJI.licenseKey()) {
    return false;
  } // Move HJI require into global namespace for Slipstream to use it


  if (!window.define) {
    window.define = hji.define;
  }

  if (!window.require) {
    window.require = window.requirejs = hji.require;
  } // Pass loaded libraries to the require js


  if (window.jQuery) {
    window.define('jquery', function () {
      return window.jQuery;
    });
  }

  if (window._) {
    window.define('underscore', function () {
      return window._;
    });
  }

  if (window.Backbone) {
    window.define('backbone', function () {
      return window.Backbone;
    });
  } // Init Slipstream


  var options = {
    license: HJI.licenseKey(),
    version: HJI.commonVar('apiVersion'),
    hooks: {
      'pre-config': function preConfig(config) {
        config.paths.national = nationalApiUrl + '/js/1.0.8';
        config.paths.sha = 'https://cdnjs.cloudflare.com/ajax/libs/jsSHA/2.0.2';
        config.paths.snap = 'https://cdnjs.cloudflare.com/ajax/libs/snap.svg/0.4.1/snap.svg'; //config.paths.i18n = 'https://cdnjs.cloudflare.com/ajax/libs/require-i18n/2.0.4/i18n.min';

        config.paths.handlebars = '//cdnjs.cloudflare.com/ajax/libs/handlebars.js/4.0.5/handlebars.min';
        config.paths.marionette = '//cdnjs.cloudflare.com/ajax/libs/backbone.marionette/2.4.4/backbone.marionette.min';
        config.bundles = {
          'slipstream/api': ['slipstream/common/model/address', 'slipstream/common/model/async', 'slipstream/common/model/coordinates', 'slipstream/common/model/response', 'slipstream/common/model/resultset', 'slipstream/common/statistics/model/interval', 'slipstream/common/statistics/util/formatter', 'slipstream/common/statistics/view/chart', 'slipstream/common/util/enumeration', 'slipstream/common/util/formatter', 'slipstream/common/util/geolocator', 'slipstream/common/util/styles', 'slipstream/common/util/types', 'slipstream/common/util/uri', 'slipstream/common/view/async', 'slipstream/common/view/loader'],
          'slipstream/areas': ['slipstream/areas/model/area', 'slipstream/areas/model/areas', 'slipstream/areas/widget/chooser', 'slipstream/areas/widget/details'],
          'slipstream/listings': ['slipstream/listings/model/listings', 'slipstream/listings/view/catalogue', 'slipstream/listings/stylesheet'],
          'slipstream/sales': ['slipstream/sales/model/sales', 'slipstream/sales/view/catalogue', 'slipstream/sales/stylesheet']
        };
      }
    }
  };

  if (HJI.commonVar('token') && HJI.commonVar('tokenExpires')) {
    options.auth = {
      token: HJI.commonVar('token'),
      expires: HJI.commonVar('tokenExpires')
    };
  }

  Slipstream.bootstrap(options);
  return Slipstream;
});


hji.define('hji-common/widgets/sales/statistics',['require', 'jquery', 'underscore', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, _, Widget) {
  /**
   * Sales Statistics Widget
   *
   * Parameters:
   *      @param data-init string             widgets/sales/statistics
   *      @param data-widgets JSON Array      ["sold-sale-price", "sold-count"]
   *      @param data-parameters JSON Object  supports all sales filters
   *          @api https://slipstream.homejunction.com/docs/ws/sales#search
   *
   * Example:
   *      <div data-init="widgets/sales/statistics"
   *          data-parameters='{"polygon":"$6e87e3e7bf04cda3ce8bdaecf473e2d8"}'
   *          data-widgets='["sold-sale-price", "sold-count"]'></div>
   *
   *
   *      Widget Object can be accessed in the DOM as follows:
   *          $('[data-init="widgets/sales/statistics"]')[0].widget;
   *
   */
  var statsWidget = function statsWidget() {
    this.widgets = ['sold-count'];
    this.options = {};
    this.loaded = [];
  };

  statsWidget.prototype = new Widget();

  statsWidget.prototype.init = function ($el, options) {
    this.$el = $el;
    this.$el[0][this.moduleName] = this;

    if (_.has(options, 'widgets')) {
      this.widgets = options.widgets;
    }

    $.extend(this.options, _.omit(options, 'widgets'));
    this.render();
  };

  statsWidget.prototype.render = function () {
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/sales'], function () {
        if (that.options.chooser) {
          that.loadChooser();
        }

        that.loadWidgets(that.options);
      });
    });
  };

  statsWidget.prototype.loadWidgets = function () {
    var that = this;
    if (that.loaded.length) return;

    for (var i in that.widgets) {
      var widget = that.widgets[i];
      var widgetWrapper = that.$el.find('.widget-wrapper.' + widget + '-widget');

      if (!widgetWrapper.length) {
        widgetWrapper = $('<div class="widget-wrapper ' + widget + '-widget"></div>');
        that.$el.append(widgetWrapper);
      }

      that.loadWidget(widget);
    }
  };

  statsWidget.prototype.loadWidget = function (widget) {
    var that = this;

    window.require(['slipstream/sales/statistics/widget/' + widget, 'slipstream/sales/statistics/model/measure', 'slipstream/common/statistics/model/interval'], function (Widget, Model, Interval) {
      /*
       * We want to display a graph of all of the sales within the last two years.
       * Compute our starting and ending date.
       */
      var today = new Date();
      var twoYearsAgo = new Date();
      twoYearsAgo.setFullYear(today.getFullYear() - 2);
      /*
       * We are going to group the sales by quarter.  Let's make sure
       * our start date begins on the first second of a quarter and our
       * ending date ends on the last second of a quarter.
       */

      var interval = Interval.MONTH;
      var startTime = Math.floor(Interval.startOf(twoYearsAgo.getTime(), interval) / 1000);
      var endTime = Math.floor(Interval.startOf(today.getTime(), interval) / 1000) - 1;
      var model = new Model();
      var defaultParams = {
        saleDate: startTime + ':' + endTime,
        measurements: 'salePrice',
        groupBy: 'saleDate',
        classifier: 'interval(month)'
      };
      model.fetch({
        parameters: _.defaults(that.options.parameters, defaultParams)
      });

      if (!that.loaded[widget]) {
        that.loaded[widget] = Widget.init({
          renderTo: that.$el.find('.widget-wrapper.' + widget + '-widget'),
          model: model
        });
      }
    });
  };

  return statsWidget;
});


hji.define('hji-common/widgets/sales/catalogue',['require', 'jquery', '_', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, _, Widget) {
  /**
   * Sales Catalogue Widget
   *
   * Renders list of sold properties and corresponding map representation with plotted markers.
   * @source https://slipstream.homejunction.com/docs/js/1.3.3/sales/widget/catalogue
   *
   * @todo extract lookBack() method into sales parameters validation utility for re-usability
   *
   * Invocation Example:
   *  <div data-init="widgets/sales/catalogue" data-parameters='{"city":"San Diego"}'></div>
   *
   *  Initialized component object can be accessed as follows:
   *   jQuery('[data-init="widgets/sales/catalogue"]')[0].widget;
   *
   * Supported data attributes:
   *
   * @param string data-init Initializes plugin by given namespace.
   * @param object data-parameters JSON Object of sales search parameters.
   *               Note: all Sales Search web service parameters are supported:
   *               @api https://slipstream.homejunction.com/docs/ws/sales#search
   *
   * Dynamic Parameters:
   *  saleDate supports lookBack() method that generates correct query for the last number of days
   *  Example:
   *  data-parameters='{"saleDate":"lookBack(60)"}' - will return sales within the last 60 days
   *
   */
  var salesWidget = function salesWidget() {
    this.options = {
      init: null,
      // component name
      parameters: {} // widget parameters

    };
    this.loaded = [];
  };

  salesWidget.prototype = new Widget();

  salesWidget.prototype.render = function () {
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/sales'], function () {
        if (_.has(that.options, 'parameters')) {
          that.loadWidget(that.options.parameters);
        }
      });
    });
  };

  salesWidget.prototype.loadWidget = function (parameters) {
    if (this.loaded.length) return;
    var that = this;
    var widgetWrapper = that.$el.find('.widget-wrapper.sales-widget');

    if (!widgetWrapper.length) {
      widgetWrapper = $('<div class="widget-wrapper sales-widget"></div>');
      that.$el.append(widgetWrapper);
    }

    window.require(['slipstream/sales/model/sales', 'slipstream/sales/widget/catalogue'], function (Sales, Catalogue) {
      // Compute the date range between 6 months ago and today
      var today = new Date();
      var then = new Date();
      then.setMonth(today.getMonth() - 6); // 6 months ago

      var range = [then.getMonth() + 1, then.getDate(), then.getFullYear()].join('/') + ':' + [today.getMonth() + 1, today.getDate(), today.getFullYear()].join('/'); // Fetch the sales

      var sales = new Sales();
      var defaultParams = {
        saleDate: range,
        sortField: 'saleDate',
        sortOrder: 'DESC',
        limit: 150
      };
      var params = that.processParams(_.defaults(parameters, defaultParams));
      sales.fetch({
        parameters: params
      }); // Display the widget

      that.loaded['catalogue'] = Catalogue.init({
        renderTo: that.$el.find('.widget-wrapper'),
        model: sales
      });
    });
  };

  salesWidget.prototype.processParams = function (params) {
    if (_.has(params, 'saleDate')) {
      // Is a lookBack?
      var pattern = /lookBack/g;

      if (pattern.test(params.saleDate)) {
        var result = params.saleDate.match(/\d+/g);
        var days = parseInt(result[0]);

        if (days) {
          var date = new Date();
          date.setDate(date.getDate() - days);
          params.saleDate = '>' + Math.round(date.getTime() / 1000);
        }
      }
    }

    return params;
  };

  return salesWidget;
});

hji.define("hji-common/widgets/sales", function(){});
