

hji.define(['require', 'jquery', 'underscore', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, _, Widget) {
  /**
   * Sales Statistics Widget
   *
   * Parameters:
   *      @param data-init string             widgets/sales/statistics
   *      @param data-widgets JSON Array      ["sold-sale-price", "sold-count"]
   *      @param data-parameters JSON Object  supports all sales filters
   *          @api https://slipstream.homejunction.com/docs/ws/sales#search
   *
   * Example:
   *      <div data-init="widgets/sales/statistics"
   *          data-parameters='{"polygon":"$6e87e3e7bf04cda3ce8bdaecf473e2d8"}'
   *          data-widgets='["sold-sale-price", "sold-count"]'></div>
   *
   *
   *      Widget Object can be accessed in the DOM as follows:
   *          $('[data-init="widgets/sales/statistics"]')[0].widget;
   *
   */
  var statsWidget = function statsWidget() {
    this.widgets = ['sold-count'];
    this.options = {};
    this.loaded = [];
  };

  statsWidget.prototype = new Widget();

  statsWidget.prototype.init = function ($el, options) {
    this.$el = $el;
    this.$el[0][this.moduleName] = this;

    if (_.has(options, 'widgets')) {
      this.widgets = options.widgets;
    }

    $.extend(this.options, _.omit(options, 'widgets'));
    this.render();
  };

  statsWidget.prototype.render = function () {
    var that = this;
    this.$el.html(this.template());
    Slipstream.ready(function () {
      window.require(['slipstream/sales'], function () {
        if (that.options.chooser) {
          that.loadChooser();
        }

        that.loadWidgets(that.options);
      });
    });
  };

  statsWidget.prototype.loadWidgets = function () {
    var that = this;
    if (that.loaded.length) return;

    for (var i in that.widgets) {
      var widget = that.widgets[i];
      var widgetWrapper = that.$el.find('.widget-wrapper.' + widget + '-widget');

      if (!widgetWrapper.length) {
        widgetWrapper = $('<div class="widget-wrapper ' + widget + '-widget"></div>');
        that.$el.append(widgetWrapper);
      }

      that.loadWidget(widget);
    }
  };

  statsWidget.prototype.loadWidget = function (widget) {
    var that = this;

    window.require(['slipstream/sales/statistics/widget/' + widget, 'slipstream/sales/statistics/model/measure', 'slipstream/common/statistics/model/interval'], function (Widget, Model, Interval) {
      /*
       * We want to display a graph of all of the sales within the last two years.
       * Compute our starting and ending date.
       */
      var today = new Date();
      var twoYearsAgo = new Date();
      twoYearsAgo.setFullYear(today.getFullYear() - 2);
      /*
       * We are going to group the sales by quarter.  Let's make sure
       * our start date begins on the first second of a quarter and our
       * ending date ends on the last second of a quarter.
       */

      var interval = Interval.MONTH;
      var startTime = Math.floor(Interval.startOf(twoYearsAgo.getTime(), interval) / 1000);
      var endTime = Math.floor(Interval.startOf(today.getTime(), interval) / 1000) - 1;
      var model = new Model();
      var defaultParams = {
        saleDate: startTime + ':' + endTime,
        measurements: 'salePrice',
        groupBy: 'saleDate',
        classifier: 'interval(month)'
      };
      model.fetch({
        parameters: _.defaults(that.options.parameters, defaultParams)
      });

      if (!that.loaded[widget]) {
        that.loaded[widget] = Widget.init({
          renderTo: that.$el.find('.widget-wrapper.' + widget + '-widget'),
          model: model
        });
      }
    });
  };

  return statsWidget;
});