

hji.define(['require', 'jquery', 'hji-common/widgets/widget', 'hji-common/slipstream'], function (require, $, Widget) {
  /**
   * School Widget
   *
   * Parameters:
   *      @param data-init string         widgets/school
   *      @param data-id string           School ID
   *      @param data-widgets JSON Array  '["details", "enrollment", "ethnicity", "gender", "summary"]'
   *
   * Example:
   *      Renders details view of the North Ranch Elementary School in Scottsdale, AZ
   *      <div data-init="widgets/school" data-id="040593001378"></div>
   *
   *      Widget Object can be accessed in the DOM as follows:
   *       var widget = $('[data-init="widgets/school"]')[0].widget;
   *
   * Manipulating widget instance:
   *      var widget = $('[data-init="widgets/school"]')[0].widget;
   *      // Setting new school ID
   *      widget.options.id = '040593001084';
   *      // Re-rendering the whole widget
   *      widget.render();
   */
  var schoolWidget = function schoolWidget() {
    this.$el = null;
    this.params = {};
    this.details = null;
    this.options = {
      id: null,
      widgets: ['details']
    };
  };

  schoolWidget.prototype = new Widget();

  schoolWidget.prototype.render = function () {
    // Reset object instance, so it can be easily re-rendered with set options.
    this.reset();
    var that = this;
    this.$el.html(this.template());
    return new Promise(function (resolve, reject) {
      Slipstream.ready(function () {
        window.require(['slipstream/schools'], function () {
          if (that.options.id && that.options.widgets.length) that.loadWidgets(that.options.id, resolve, reject);
        });
      });
    });
  };

  schoolWidget.prototype.loadWidgets = function (id, resolve, reject) {
    var that = this;

    for (var i in that.options.widgets) {
      var widget = that.options.widgets[i];
      var widgetWrapper = that.$el.find('.widget-wrapper.' + widget + '-widget');

      if (!widgetWrapper.length) {
        widgetWrapper = $('<div class="widget-wrapper ' + widget + '-widget"></div>');
        that.$el.first('.slipstream-areas').append(widgetWrapper);
      }

      that.loadWidget(widget, resolve, reject);
    }
  };

  schoolWidget.prototype.loadWidget = function (widget, resolve, reject) {
    var that = this;

    window.require(['slipstream/schools/view/' + widget], function (Details) {
      var details = new Details();
      details.model.fetch({
        parameters: {
          id: that.options.id,
          details: true
        }
      });
      details.render().$el.appendTo(that.$el.find('.details-wrapper'));
      resolve();
    });
  };

  schoolWidget.prototype.renderWidget = function (widget) {
    var that = this;

    window.require(['slipstream/schools/view/' + widget], function (Widget) {
      if (!that.loaded[widget]) {
        that.loaded[widget] = Widget.render().$el.appendTo(that.$el.find('.details-wrapper'));
      }
    });
  };

  schoolWidget.prototype.template = function () {
    var template = $('<div class="slipstream slipstream-reset">').append($('<div class="slipstream-school">').append($('<div class="details-wrapper">')));
    if (this.options.chooser) template.find('.slipstream-school').prepend('<div class="chooser-wrapper">');
    return template;
  };
  /**
   * Resets widget instance so it can be re-rendered.
   */


  schoolWidget.prototype.reset = function () {
    this.model = null;
    this.loaded = [];
  };

  return schoolWidget;
});