

/**
 *  Instantiates RequireJS and all registered plugin's main.js (main-built.js)
 *  files asynchronously.
 *  Loaded by WP via wp_enqueue_script().
 *  All main/main-built js files have to be referenced in the
 *  hjiRequireJsConfig in paths as moduleName/main: path/to/main-built.js
 *  This can be done via PHP:
 *  \hji\Membership::getInstance()
 *      ->requireJs->addPath('moduleName/main', 'path/to/main-built.js');
 *  Use main.js for development and main-built.js for production.
 */
window.HJI = function () {
  var hji = {
    debug: false,
    vars: {},
    set: function set(name, value) {
      this.vars[name] = value;
    },
    get: function get(name) {
      return typeof this.vars[name] !== 'undefined' ? this.vars[name] : -1;
    },
    loadCss: function loadCss(url) {
      var link = document.createElement("link");
      link.type = "text/css";
      link.rel = "stylesheet";
      link.href = url;
      document.getElementsByTagName("head")[0].appendChild(link);
    },
    logError: function logError(message, method) {
      if (console && console.error) {
        if (method) {
          console.error(message + ' in ' + method);
        } else {
          console.error(message);
        }
      }
    },
    log: function log(message) {
      if (this.debug) console.log(message);
    },
    ajaxUrl: function ajaxUrl() {
      return this.commonVar('ajaxUrl');
    },
    licenseKey: function licenseKey() {
      return this.commonVar('licenseKey');
    },
    siteUrl: function siteUrl() {
      return this.commonVar('siteUrl');
    },
    setup: function setup() {
      this.vars = typeof hjiSettings != 'undefined' ? hjiSettings : {};
      this.debug = this.commonVar('debug') === true ? true : false;
    },
    commonVar: function commonVar(varName) {
      var common = this.get('common');

      if (common && common[varName]) {
        return common[varName];
      }

      return null;
    },
    utils: {}
  };
  hji.setup();
  document.addEventListener("DOMContentLoaded", function () {
    if (window.HJI.debug === true) {
      console.log('%c HJI DEBUG MODE IS ON', 'font-weight: bold; color: green;');
      var query = decodeURIComponent(location.search.substring(1));

      if (query) {
        console.log('%c URL Query:', 'color: green;');
        var queryObj = JSON.parse('{"' + decodeURI(query).replace(/"/g, '\\"').replace(/&/g, '","').replace(/=/g, '":"') + '"}');
        queryObj.location = queryObj.location ? JSON.parse(queryObj.location) : '';
        console.log(queryObj);
      }
    }
  });
  return hji;
}();

HJI.Membership = function () {
  var __obj = {};
  return __obj;
}();

if (!String.prototype.includes) {
  String.prototype.includes = function (search, start) {
    if (typeof start !== 'number') {
      start = 0;
    }

    if (start + search.length > this.length) {
      return false;
    } else {
      return this.indexOf(search, start) !== -1;
    }
  };
} // Load RequireJS


(function ($) {
  var readyToRoll = function readyToRoll() {
    var config = typeof hjiRequireJsConfig != 'undefined' ? hjiRequireJsConfig : {};

    hji.require.config(config);

    if (window.jQuery) {
      hji.define('jquery', function () {
        return window.jQuery;
      });
    }

    if (window._) {
      hji.define('underscore', function () {
        return window._;
      });
    }

    if (window.Backbone) {
      hji.define('backbone', function () {
        return window.Backbone;
      });
    } // Instantiate all main.js files
    // that were set in config.paths by HJI plugins.


    if (typeof config.paths != 'undefined') {
      for (i in config.paths) {
        if (i.indexOf('/main') > -1) {
          hji.require([i + '']);
        }
      }
    }
  };

  readyToRoll();
})(jQuery);